<?php
namespace Teknisa\DF\Controllers;

use Teknisa\DF\Services\ContabilizacaoService;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;

class ContabilizacaoController {

    protected $contabilizacaoService;

    public function __construct(ContabilizacaoService $contabilizacaoService) {
        $this->contabilizacaoService = $contabilizacaoService;
    }

    /*
     * Função para buscar as partidas e os rateios de um dado lançamento.
     */
    public function getContabilizacaoLancamentos(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $lancamentos = $request->getDataSet()->getRows();

            // Busca os dados do lançamento.
            $lancamento = $this->contabilizacaoService->getContabilizacaoLancamentos($lancamentos);
            
            // Cria os respectivos DataSets.
            $partidasDataSet = new DataSet('partidasContabilizacao', $lancamento["partidas"]);
            $rateiosDataSet = new DataSet('rateiosContabilizacao', $lancamento["rateios"]);

            // Adiciona os DataSets no response da requisição.
            $response->addDataSet($partidasDataSet);
            $response->addDataSet($rateiosDataSet);
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function verificaErroContab(DTO\Request $request, DTO\Response $response) {
        try {
            $dataSet = $request->getDataSet()->getRows();
            
            $retorno = $this->contabilizacaoService->verificaErroContab($dataSet, $response);
            
            if($retorno){
                $this->contabilizacaoService->gravaLancContab($dataSet);
                $response->addDataSet(new DataSet('response', [true]));
            }

        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
}