<?php

namespace Teknisa\DF\Controllers;

use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Row;
use Zeedhi\Framework\DataSource\DataSet;

class ParametrosNfe extends \Teknisa\DF\Controllers\TeknisaCrud {
    
    protected $dataSourceName = 'paramgeral_paramintegra';
    protected $crudService;
    protected $parameterBag;
    protected $environment;
    protected $nameProvider;
  
    public function __construct($crudService, $parameterBag, $environment, $nameProvider) {
        $this->crudService  = $crudService;
        $this->parameterBag = $parameterBag;
        $this->environment  = $environment;
        $this->nameProvider = $nameProvider;
    }
  
    public function find(DTO\Request\Filter $request, DTO\Response $response) {
        parent::find($request, $response);
        
        $dataSet = $response->getDataSets()[0];
        $row = $dataSet->getRows()[0];

        if($row["CDPARAMINT"] == null) {
            
            $dataSource = $this->nameProvider->getDataSourceByName('paramintegra');
            
            $rowsToSave = array("CDPARAMINT"    => "1",
                               "IDPERPRNVINC"   => "N",
                               "IDPERIMPSPRAF"  => "N",
                               "IDIMPORTXMLNFE" => "N",
                               "__is_new"       => true);
                               
            $this->adicionaCamposAuditoria($rowsToSave);
            $this->getRowWithDefaultColumnsProperties($rowsToSave, $dataSource);
            $rowToSave = $this->crudService->save("paramintegra", array($rowsToSave));
             
            parent::find($request, $response);
        }
    
     }
     
    public function save(DTO\Request $request, DTO\Response $response) {
        
        $this->dataSourceName = 'paramintegra';
        parent::save($request, $response);
        
        $this->dataSourceName = 'paramgeral';
        parent::save($request, $response);
        
    }
  
    protected function adicionaCamposAuditoria(&$row) {
        $currentTime = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
        $row['NRORG'] = (isset($row['NRORG']) && !empty($row['NRORG'])) ? $row['NRORG'] : $this->environment->getNrOrgTrab();
        $row['IDATIVO'] = isset($row['IDATIVO']) ? ($row['IDATIVO'] ?: 'S') : 'S';
        $row['DTULTATU'] = $currentTime;
        if($row["__is_new"] == true) {
            $row['DTINCLUSAO'] = $currentTime;
            $row['NRORGINCLUSAO'] = $this->environment->getNrOrgTrab();
            $row['CDOPERINCLUSAO'] = $this->environment->getCdOperador();
        }
        $row['NRORGULTATU']  = $this->environment->getNrOrgTrab();
        $row['CDOPERULTATU'] = $this->environment->getCdOperador();
        $row['DTULTATU']     = $currentTime;
    }
    
    
    private function getRowWithDefaultColumnsProperties(&$row, $dataSource) {
        if (method_exists($dataSource, 'getColumnsProperties')) {
            $columnsProperties = $dataSource->getColumnsProperties();

            if (isset($columnsProperties)) {
                foreach ($columnsProperties as $columnProperty) {
                    if (!\Zeedhi\Framework\Util\Functions::arrayKeyExists($columnProperty['COLUMN_NAME'], $row)) {
                        if ($columnProperty['NULLABLE'] == 'Y') {
                            $row[$columnProperty['COLUMN_NAME']] = $this->getDataDefaultValue($columnProperty);
                        } else if ($columnProperty['COLUMN_NAME'] == 'CDFILIAL') {
                            $row[$columnProperty['COLUMN_NAME']] = $this->environment->getCdFilial();
                        } else if ($columnProperty['NULLABLE'] == 'N') {
                            if (isset($columnProperty['DATA_DEFAULT'])){
                                $row[$columnProperty['COLUMN_NAME']] = $this->getDataDefaultValue($columnProperty);
                            } else {
                                throw new \Exception("Column ".$columnProperty['COLUMN_NAME']." cannot be null and does not have a default value! Set a value for this column.", 1);
                            }
                        }
                    }
                }
            }
        }

        return $row;
    }
    
    
    private function getDataDefaultValue($columnProperty) {
        if ($columnProperty['DATA_DEFAULT'] == null) {
            return null;
        } else {
            switch ($columnProperty['DATA_TYPE']) {
                case 'DATE':
                    return $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
                    break;
                case 'NUMBER':
                    return intval($columnProperty['DATA_DEFAULT']);
                    break;
                case 'FLOAT':
                    return floatval($columnProperty['DATA_DEFAULT']);
                    break;
                default:
                    return $columnProperty['DATA_DEFAULT'];
                    break;
            }
        }
    }
}