<?php

namespace Teknisa\DF\Factories;

use Doctrine\ORM\EntityManager;

class AprovacaoLancamentosFactory {
    private $entityManager;

    public function __construct(EntityManager $entityManager, $environment)
    {
        $this->entityManager = $entityManager;
        $this->environment   = $environment;
    }

    public function getTipoContaPagarProd($cdfilial, $nrlanctonf, $nrorg)
    {
        $sql = <<<SQL
        SELECT P3.CDPRODUTO, P3.CDARVPROD, P4.CDARVPROD AS CDNIVEL, P4.CDTPCTPAGAR,
               ROUND(A.VRBRUTITEM,2) AS VRBRUTITEM
          FROM ITEMPRODUTO A, PRODUTO P3, PRODUTO P4
         WHERE A.CDFILIAL = :P_CDFILIAL
           AND A.NRLANCTONF = :P_NRLANCTONF
           AND A.NRORG = :P_NRORG
           AND A.CDPRODUTO = P3.CDPRODUTO
           AND A.NRORG = P3.NRORG
           AND (INSTR(P3.CDARVPROD,NVL(RTRIM(P4.CDARVPROD),P3.CDARVPROD)) = 1)
           AND P4.CDTPCTPAGAR IS NOT NULL
           AND P4.CDARVPROD = (SELECT MAX(P1.CDARVPROD)
                         FROM ITEMPRODUTO IT,  PRODUTO P, PRODUTO P1
                        WHERE IT.CDFILIAL = A.CDFILIAL
                                  AND IT.NRLANCTONF = A.NRLANCTONF
                                  AND IT.CDPRODUTO = A.CDPRODUTO
                                  AND IT.NRORG = A.NRORG
                      AND IT.CDPRODUTO = P.CDPRODUTO
                      AND IT.NRORG = P.NRORG
                      AND (INSTR(P.CDARVPROD,NVL(RTRIM(P1.CDARVPROD),P.CDARVPROD)) = 1)
                      AND P1.CDTPCTPAGAR IS NOT NULL)
         ORDER BY P3.CDARVPROD
SQL;
        $params = array(
            "P_NRLANCTONF" => $nrlanctonf,
            "P_CDFILIAL" => $cdfilial,
            "P_NRORG" => $nrorg
        );

        return $this->entityManager->getConnection()->fetchAll($sql, $params);
    }
    

    public function getAutoForne($cdfilial, $nrlanctonf, $nrorg){
        $sql = <<<SQL
         SELECT MAX(T.CDTPCTPAGAR) AS CDTPCTPAGAR,
                   MAX(T.CDCLASSFINA) AS CDCLASSFINA
            FROM TIPOCTPAGAR T, AUTOFORNE A, NFRELACAF N
            WHERE N.CDFILIAL    = :P_CDFILIAL   AND
                  N.NRLANCTONF  = :P_NRLANCTONF AND
                  N.NRORG       = :P_NRORG      AND
                  A.CDFILDEST   = N.CDFILDEST   AND
                  A.CDFILLANC   = N.CDFILLANC   AND
                  A.NRAUTOFORN  = N.NRAUTOFORN  AND
                  A.NRORG       = N.NRORG       AND
                  A.CDTPCTPAGAR = T.CDTPCTPAGAR AND
                  A.NRORG       = T.NRORG
SQL;

        $params = array(
            "P_NRLANCTONF" => $nrlanctonf,
            "P_CDFILIAL" => $cdfilial,
            "P_NRORG" => $nrorg
        );

        try {
            return $this->entityManager->getConnection()->fetchAssoc($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function verificaTipoContaPagar($cdfilial, $nrlanctonf, $nrorg)
    {
        $sql = <<<SQL
         SELECT
               MIN(C.QTDIASVENPAG), C.CDTPCTPAGAR, T.CDCLASSFINA
            FROM
               CONTAPAGAR C, TIPOCTPAGAR T
            WHERE
               (C.CDFILIAL = :P_CDFILIAL) AND
               (C.NRLANCTONF = :P_NRLANCTONF) AND
               (C.NRORG = :P_NRORG)         AND
               (C.CDTPCTPAGAR IS NOT NULL) AND
               (C.CDTPCTPAGAR = T.CDTPCTPAGAR) AND
               (C.NRORG = T.NRORG)
            GROUP BY
               C.CDTPCTPAGAR, T.CDCLASSFINA
SQL;

        $params = array(
            "P_NRLANCTONF" => $nrlanctonf,
            "P_CDFILIAL"   => $cdfilial,
            "P_NRORG"      => $nrorg
        );

        try {
            return $this->entityManager->getConnection()->fetchAssoc($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function getNumeros($nrnotafisc, $lancamento, $cdserie, $cdmoddocfisc, $identrsaidop, $nrinsjurfili){
        if ($identrsaidop == 'S') {
            $sql = "SELECT 1
                    FROM
                        NOTAFISCAL NF JOIN FILIAL F ON (NF.CDFILIEMIT = F.CDFILIAL)
                    WHERE LPAD(NF.NRNOTAFISC,20,'0') = LPAD(:P_NRNOTAFISC,20,'0')
                      AND NF.CDFILIAL || NF.NRLANCTONF <>:P_LANCAMENTO
                      AND NF.IDSISTEMA NOT IN (
                            'REC',
                            'TMP',
                            'ITG'
                        )
                      AND NF.CDSERIE = :P_CDSERIE
                      AND NF.CDMODDOCFISC = :P_CDMODDOCFISC
                      AND NF.IDENTRSAIDOP = :P_IDENTRSAIDOP
                      AND F.NRINSJURFILI = :P_NRINSJURFILI";
        }
        else {
            $sql = "SELECT 1
                    FROM
                        NOTAFISCAL NF JOIN FILIAL F ON (NF.CDFILIRECE = F.CDFILIAL)
                    WHERE LPAD(NF.NRNOTAFISC,20,'0') = LPAD(:P_NRNOTAFISC,20,'0')
                      AND NF.CDFILIAL || NF.NRLANCTONF <>:P_LANCAMENTO
                      AND NF.IDSISTEMA NOT IN (
                            'REC',
                            'TMP',
                            'ITG'
                        )
                      AND NF.CDSERIEENTR = :P_CDSERIE
                      AND NF.CDMODDOCFISC = :P_CDMODDOCFISC
                      AND NF.IDENTRSAIDOP = :P_IDENTRSAIDOP
                      AND F.NRINSJURFILI = :P_NRINSJURFILI";
        }
        $params = array(
            "P_NRNOTAFISC"   => $nrnotafisc,
            "P_LANCAMENTO"   => $lancamento,
            "P_CDSERIE"      => $cdserie,
            "P_CDMODDOCFISC" => $cdmoddocfisc,
            "P_IDENTRSAIDOP" => $identrsaidop,
            "P_NRINSJURFILI" => $nrinsjurfili
        );

        return $this->entityManager->getConnection()->fetchAll($sql, $params);
    }
    
    public function getImposBloqDtemissao($cdfilial, $nrlanctonf, $cdempresa, $dtemissao){
        $sql = "SELECT M.NRSEQUMAPAPU, M.IDBLOQMAPAPU, M.CDIMPOSTO
                FROM NOTAFISCAL N, IMPOSTONF I, IMPOSTO IE, MAPAAPURAC M
                WHERE N.CDFILIAL   = :P_CDFILIAL
                  AND N.NRLANCTONF = :P_NRLANCTONF
                  AND N.CDFILIAL = I.CDFILIAL
                  AND N.NRLANCTONF = I.NRLANCTONF
                  AND M.CDEMPRESA  = :P_CDEMPRESA
                  AND I.CDIMPOSTO = IE.CDIMPOSTO
                  AND IE.CDIMPOSTONFE = M.CDIMPOSTO
                  AND I.IDIMPOSTONF = '1'
                  AND ((M.CDINSCESTA = N.CDINSCESTA AND IE.IDORGARRECAD = 'E') 
                       OR (M.CDINSCMUNI = N.CDINSCMUNI AND IE.IDORGARRECAD = 'M') 
                       OR ((IE.IDORGARRECAD = 'F' OR IE.IDORGARRECAD IS NULL))) 
                  AND (:P_DATANF BETWEEN M.DTINIPERIODO AND M.DTFIMPERIODO)
                  AND M.IDBLOQMAPAPU = 'S'
                ORDER BY M.CDIMPOSTO, M.NRSEQUMAPAPU";
        $params = array(
            "P_CDFILIAL"   => $cdfilial,
            "P_NRLANCTONF" => $nrlanctonf,
            "P_CDEMPRESA"  => $cdempresa,
            "P_DATANF"     => $dtemissao
        );
        
        return $this->entityManager->getConnection()->fetchAll($sql, $params);
    }
    
    public function getContReceber($filial, $lancamento, $organizacao){
        $sql = <<<SQL
                 SELECT TR.CDTPCTRECEB, TR.CDCLASSFINA
                    FROM (SELECT MAX(CDARVPROD) AS CDARVPROD
                          FROM ITEMPRODUTO I, PRODUTO P
                          WHERE CDFILIAL    = :P_CDFILIAL AND
                                NRLANCTONF  = :P_NRLANCTONF AND
                                I.NRORG     = :P_NRORG AND
                                I.CDPRODUTO = P.CDPRODUTO AND
                                I.NRORG     = P.NRORG
                          GROUP BY VRTOTAITEM
                          HAVING VRTOTAITEM = (SELECT MAX(VRTOTAITEM)
                                               FROM ITEMPRODUTO
                                               WHERE CDFILIAL    = :P_CDFILIAL AND
                                                     NRLANCTONF  = :P_NRLANCTONF AND
                                                     NRORG       = :P_NRORG)) IT,
                          PRODUTO PR , TIPOCTRECEBER TR
                    WHERE TR.CDTPCTRECEB = PR.CDTPCTRECEB AND
                          PR.CDPRODUTO = (SELECT MAX(CDPRODUTO)
                                          FROM TIPOCTRECEPROD C
                                          WHERE (INSTR(IT.CDARVPROD,RTRIM(C.CDARVPROD)) = 1))
SQL;
        $params = array(
            "P_CDFILIAL"    => $filial,
            "P_NRLANCTONF" => $lancamento,
            "P_NRORG" => $organizacao
        );

        try{
            return $this->entityManager->getConnection()->fetchArray($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function consisteFiscal($cdfilial, $nrlanctonf)
    {
        $sql = <<<SQL
    SELECT CDIMPOSTO, NRSEQITPROD, NRSEQCONSISTE, DSCONSISTENCIA, CDARVPROD, NMPRODUTO,
       CDREGRAIT
  FROM TABLE(CONSISTE_FISCAL(:P_CDFILIAL, :P_NRLANCTONF))
SQL;
        try{

            $params = array(
                "P_CDFILIAL" => $cdfilial,
                "P_NRLANCTONF" => $nrlanctonf
            );
            return $this->entityManager->getConnection()->fetchAll($sql, $params);
        }catch(\Exception $e){
            throw $e;
        }
    }
      
    public function getParamManifesta() {
    	$sql = "SELECT IDCIENCIAEMISS, IDCONFIRMAEMISS
    			FROM PARAMMANIFESTA
    			WHERE  ((IDCIENCIAEMISS = '3') OR  (IDCONFIRMAEMISS = '3'))";
       
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute();
		return $std->fetchAll(); 
    }
    
    public function getNotaDestinada($nrAcessoNfe) {
		$sql = "SELECT NVL(IDSTATUSMANIF, '0') IDSTATUSMANIF
		        FROM NOTADESTINADA
		       WHERE NRACESSONFE = :P_NRACESSONFE";
		
		$params = array("P_NRACESSONFE"		=> $nrAcessoNfe);
		       
		$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetchAll(); 
	}
	
	public function existeNotaBCMensal($cdfilial, $nrlanctonf) {
	    
	    $sql = "SELECT CDFILIAL,
                  NRLANCTONF
                FROM NOTABCMENSAL
                WHERE CDFILIALIT = :P_CDFILIALIT
                AND NRLANCTONFIT = :P_NRLANCTONFIT";
                
		$params = array("P_CDFILIALIT"		=> $cdfilial,
		                "P_NRLANCTONFIT"    => $nrlanctonf);
		       
		$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
         
		$nota = $std->fetch();
		
	    return !!$nota;
	}
	
	public function deleteTrabimposNfTot($cdfilial, $nrlanctonf) {
	    $sql = "DELETE TRABIMPOSNFTOT
                WHERE CDFILIMPTOT = :P_CDFILIAL
                AND NRLANCIMPTOT  = :P_NRLANCTONF";
                
        $params = array("P_CDFILIAL"		=> $cdfilial,
		                "P_NRLANCTONF"    => $nrlanctonf);
		       
		$std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
	}
	
	public function deleteTrabimposNfTotImposto($cdfilial, $nrlanctonf, $cdimpostoit) {
	    $sql = "DELETE TRABIMPOSNFTOT
                WHERE CDFILIMPTOT = :P_CDFILIAL
                AND NRLANCIMPTOT  = :P_NRLANCTONF
                AND CDIMPOSTOT    = :P_CDIMPOSTOT";
                
        $params = array("P_CDFILIAL"		=> $cdfilial,
		                "P_NRLANCTONF"      => $nrlanctonf,
		                "P_CDIMPOSTOT"      => $cdimpostoit);
		       
		$std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
	}
    
    public function insereTrabimposNfTot($cdfilial, $nrlanctonf, $cdimpostoit, $vrimpostoit) {

        $sql = "INSERT
                INTO TRABIMPOSNFTOT
                  (
                    CDFILIMPTOT,
                    NRLANCIMPTOT,
                    CDIMPOSTOT,
                    VRIMPOSTOT,
                    VRBCALIMPTOT,
                    NRORG
                  )
                  VALUES
                  (
                    :P_CDFILIAL,
                    :P_NRLANCTONF,
                    :P_CDIMPOSTOT,
                    (0 + :P_VRIMPOSTOT),
                    0,
                    :P_NRORG
                  )";
                  
        $params = array("P_CDFILIAL"		=> $cdfilial,
		                "P_NRLANCTONF"      => $nrlanctonf,
		                "P_CDIMPOSTOT"      => $cdimpostoit,
		                "P_VRIMPOSTOT"      => $vrimpostoit,
		                "P_NRORG"           => $this->environment->getNrOrgTrab());
		       
		$std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
    }
	
	public function getProdutosNota($cdfilial, $nrlanctonf) {
	    $sql = "SELECT CDARVPROD,
                  NVL(VRTOTAITEM,0) VRTOTAITEM,
                  NVL(VRDESCITEM,0) VRDESCITEM,
                  PRODUTO.CDPRODUTO,
                  PRODUTO.NMPRODUTO,
                  NRSEQITPROD,
                  VRCONTITEM,
                  IDORIGEMPROD
                FROM ITEMPRODUTO,
                  PRODUTO
                WHERE NRLANCTONF      = :P_NRLANCTONF
                AND CDFILIAL          = :P_CDFILIAL
                AND PRODUTO.CDPRODUTO = ITEMPRODUTO.CDPRODUTO";
                
        $params = array("P_CDFILIAL"		=> $cdfilial,
		                "P_NRLANCTONF"      => $nrlanctonf);
		       
		$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetchAll(); 
	}
	
	public function getImpoInfoMan($cdfilial, $nrlanctonf, $nrseqitprod) {
	    $sql = "SELECT IT.CDIMPOSTO,
                  VRIMPOSTOIT,
                  VRALIQITEM,
                  IDVALORINFIT,
                  VRPERREDBCIT,
                  VRBASECALCIT,
                  VRADICALIQFECP,
                  IDPRODNOTA,
                  VRADICFECP
                FROM ITEMPROIMPOS IT, IMPOSTO IM
                WHERE CDFILIAL   = :P_CDFILIAL
                AND NRLANCTONF   = :P_NRLANCTONF
                AND NRSEQITPROD  = :P_NRSEQITPROD
                AND NRITPROIMPOS = '1'
                AND IDVALORINFIT = 'S'
                AND IT.CDIMPOSTO = IM.CDIMPOSTO";
                
        $params = array("P_CDFILIAL"		=> $cdfilial,
		                "P_NRLANCTONF"      => $nrlanctonf, 
		                "P_NRSEQITPROD"     => $nrseqitprod);
                
    	$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetchAll(); 
	}
	
	public function getAliqUniFis($cdfilial, $nrlanctonf) {
	    
	    $sql = "SELECT IM.CDIMPOSTO,
                  IM.SGIMPOSTO,
                  IM.IDTRATIMPO,
                  IM.IDPREIMPENTR,
                  I.CDFISCTEMP,
                  NVL(AL.VRALIQUTMP, 0) AS VRALIQUOTA,
                  IM.IDPRODNOTA,
                  NVL(AL.VRADICALIQFECP, 0) AS VRADICALIQFECP
                FROM INCITEMP I,
                  IMPOSTO IM,
                  ALIQTEMPPROD AL
                WHERE I.CDIMPOTEMP = IM.CDIMPOSTO
                AND I.NRINCITEMP   = :P_CDFILIAL
                  ||:P_NRLANCTONF
                AND I.NRINCITEMP = AL.NRALIQTEMP(+)
                AND I.CDIMPOTEMP = AL.CDIMPOTEMP(+)";
	    
        $params = array("P_CDFILIAL"		=> $cdfilial,
		                "P_NRLANCTONF"      => $nrlanctonf);
                
    	$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetchAll(); 
	}
	
	public function bloqueiaOperadorNotaFiscal($cdfilial, $nrlanctonf) {
	    $sql = "SELECT CDFILIAL
                FROM NOTAFISCAL
                WHERE CDFILIAL = :P_CDFILIAL
                AND NRLANCTONF = :P_NRLANCTONF FOR UPDATE NOWAIT";	    
                
        $params = array("P_CDFILIAL"		=> $cdfilial,
		                "P_NRLANCTONF"      => $nrlanctonf);
        
    	$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetchAll(); 
	}
	
	public function existeMapaApuradoBloqueado($cdfilial, $nrlanctonf) {
	    $sql = "SELECT MAX(I.NRSEQUMAPAPU) AS NRSEQUMAPAPU,
                  MAX(NRSEQMAPOIMP)        AS NRSEQMAPOIMP,
                  MAX(MP.NMMAPAAPUR)       AS NMMAPAAPUR
                FROM IMPOSTONF I,
                  MAPAAPURAC MP,
                  IMPOSTO II
                WHERE I.CDFILIAL     = :P_CDFILIAL
                AND I.NRLANCTONF     = :P_NRLANCTONF
                AND ((I.NRSEQUMAPAPU = MP.NRSEQUMAPAPU)
                OR (MP.NRSEQUMAPAPU  = I.NRSEQMAPOIMP))
                AND II.CDIMPOSTONFE  = MP.CDIMPOSTO
                AND MP.IDBLOQMAPAPU  = 'S'";
        
        $params = array("P_CDFILIAL"		=> $cdfilial,
		                "P_NRLANCTONF"      => $nrlanctonf);
		                
		$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
         
		$mapa = $std->fetch();
		
	    return !!$mapa["NRSEQUMAPAPU"];
	}
	
	public function existeMapaSimplesBloqueado($cdfilial, $nrlanctonf) {
	    $sql = "SELECT N.CDFILIAL
                FROM DETALHEAPURSN D,
                  MAPAAPURACSN M,
                  SIMPLESNACIONAL S,
                  NOTAFISCAL N
                WHERE D.NRSEQUMAPAPU    = M.NRSEQUMAPAPU
                AND S.CDSIMPLESNACIONAL = D.CDSIMPLESNACIONAL
                AND N.CDFILIAL          = :P_CDFILIAL
                AND N.NRLANCTONF        = :P_NRLANCTONF
                AND N.NRSEQUMAPAPUSN    = M.NRSEQUMAPAPU
                AND M.IDBLOQMAPAPU      = 'S'";
                
        $params = array("P_CDFILIAL"		=> $cdfilial,
		                "P_NRLANCTONF"      => $nrlanctonf);
                
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
         
		$mapa = $std->fetch();
		
	    return !!$mapa;
	}
	
	public function validaDataLancamento($cdEmpresa, $dtEmis) {
	    $sql = "SELECT CDEMPRESA
                FROM CONTOPECTBL
                WHERE DTINICLANC <= :P_DTEMIS
                AND DTFINALANC   >= :P_DTEMIS
                AND CDEMPRESA     = :P_EMPRESA";
                
        $params = array("P_EMPRESA"         => $cdEmpresa,
		                "P_DTEMIS"          => $dtEmis    );
                
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
         
		$dataValida = $std->fetch();
		
	    return !!$dataValida;
	}
	
	public function existeMapaLucroPresumido($nrseqmapaapu) {
	    $sql = "SELECT NMMAPAAPUR
                FROM MAPAAPURAC
                WHERE NRSEQUMAPAPU = :P_NRSEQUMAPAPU
                AND IDBLOQMAPAPU   = 'S'";
                
        $params = array("P_NRSEQUMAPAPU"         => $nrseqmapaapu);
                
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
         
		$mapa = $std->fetch();
		
	    return !!$mapa;
	}
	
	public function atualizaCFOPNotaFiscal($cdfilial, $nrlanctonf) {
	    $sql = "UPDATE NOTAFISCAL
                SET CDFISOPENTF = NULL,
                  DSCDFISOPENF  = NULL
                WHERE (CDFILIAL = :P_CDFILIAL)
                AND (NRLANCTONF = :P_NRLANCTONF)";
                
        $params = array("P_CDFILIAL"	  => $cdfilial,
		                "P_NRLANCTONF"    => $nrlanctonf);
		       
		$std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
	}
	
	public function operacaoInterEstadual($cdfilial, $cdempresa, $cdinscesta, $cdcliente, $cdforneced, $sgestado) {
	    $sql = "SELECT INSC.SGESTADO
                FROM
                  (SELECT SGESTADO
                  FROM ENDEFORN
                  WHERE IDTPENDEFORN = 'P'
                  AND CDFORNECED     =
                    (SELECT CDFORNECED FROM FORNECEDOR WHERE CDFORNECED = :P_CDFORNECED
                    )
                  UNION ALL
                  SELECT SGESTADO
                  FROM ENDECLIE
                  WHERE IDTPENDECLIE = 'P'
                  AND CDCLIENTE      =
                    (SELECT CDCLIENTE FROM CLIENTE WHERE CDCLIENTE = :P_CDCLIENTE
                    )
                  UNION ALL
                  SELECT SGESTADO
                  FROM ENDEFILI
                  WHERE IDTPENDEFILI = 'P'
                  AND CDFILIAL       = :P_CDFILIAL
                  ) ORIG,
                  (SELECT SGESTADO
                  FROM INSCRESTAD
                  WHERE CDINSCESTA               = :P_CDINSCESTA
                  AND CDEMPRESA                  = :P_CDEMPRESA
                  AND NVL(:P_SGESTADO, SGESTADO) = SGESTADO
                  ) INSC
                WHERE ORIG.SGESTADO = INSC.SGESTADO";
                
        $params = array("P_CDFORNECED"	  => $cdforneced,
                        "P_CDCLIENTE"     => $cdcliente,
                        "P_CDFILIAL"      => $cdfilial,
                        "P_CDINSCESTA"    => $cdinscesta,
                        "P_CDEMPRESA"     => $cdempresa,
                        "P_SGESTADO"      => $sgestado); 
                        
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        
		$interEstadual = $std->fetch();
		
	    return !!$interEstadual;
	}
	
	public function getImpostoObrigatorio($cdtipooper, $identrsaidop, $cdfilial) {
	    $sql = "SELECT OB.CDIMPOSTO, I.NMIMPOSTO
                FROM IMPOBRIGTPOPER OB, IMPOSTO I, 
                  (SELECT DECODE(:P_IDENTRSAIDOP, 'E', NVL(IDUTCTRLFISENTR, 'N'), NVL(IDUTCTRLFISSAID, 'N')) IDUTCTRLFISFILIAL
                  FROM PARAMFILIAL
                  WHERE CDFILIAL = :P_CDFILIAL
                  ) PF ,
                  (SELECT DECODE(:P_IDENTRSAIDOP, 'E', NVL(PG.IDUTCTRLFISENTR, 'N'), NVL(PG.IDUTCTRLFISSAID, 'N')) IDUTCTRLFISGERAL
                  FROM PARAMGERAL PG
                  WHERE CDPARGERAL = '0001'
                  ) PG
                WHERE CDTIPOOPER           = :P_CDTIPOOPER
                AND IDENTRSAIDOP           = :P_IDENTRSAIDOP
                AND OB.CDIMPOSTO           = I.CDIMPOSTO
                AND ((PF.IDUTCTRLFISFILIAL = 'S')
                OR (PG.IDUTCTRLFISGERAL    = 'S'))";
                
        $params = array("P_CDTIPOOPER"	  => $cdtipooper,
                        "P_IDENTRSAIDOP"  => $identrsaidop,
                        "P_CDFILIAL"      => $cdfilial); 
                        
      	$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetchAll(); 
	}
	
	public function deleteTrabitproimpos($cdfilial, $nrlanctonf, $nrseqimpoit, $cdimpostoit, $nrseqitpro) {
	    $sql = "DELETE
                FROM TRABITPROIMPOS
                WHERE CDFILIMPOIT = :P_CDFILIMPOIT
                AND NRLANCIMPOIT  = :P_NRLANCIMPOIT
                AND NRSEQIMPOIT   = :P_NRSEQIMPOIT
                AND CDIMPOSTOIT   = :P_CDIMPOSTOIT
                AND NRSEQITPRO    = :P_NRSEQITPRO";
                
         $params = array("P_CDFILIMPOIT"	    => $cdfilial,
                        "P_NRLANCIMPOIT"        => $nrlanctonf,
                        "P_NRSEQIMPOIT"         => $nrseqimpoit,
                        "P_CDIMPOSTOIT"         => $cdimpostoit,
                        "P_NRSEQITPRO"          => $nrseqitpro); 
                        
      	$std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
	}
	
	public function insertTrabitproimpos($cdfilimpoit, $nrlancimpoit, $nrseqimpoit, $cdimpostoit, $nrseqitpro, $vraliqimpoit, $vrredbcimpoit, $vrbcalcimpoit,
                                        $vrimposto, $idvalorinfo, $vradicaliqfecp, $vradicfecp) {
	    $sql = "INSERT
                    INTO TRABITPROIMPOS
                      (
                        CDFILIMPOIT,
                        NRLANCIMPOIT,
                        NRSEQIMPOIT,
                        CDIMPOSTOIT,
                        NRSEQITPRO,
                        VRALIQIMPOIT,
                        VRCONTIMPOIT,
                        VRREDBCIMPOIT,
                        VRBCALCIMPOIT,
                        VRPDEDIMPOIT,
                        VRIMPOSIT,
                        IDVALORINFO,
                        VRADICALIQFECP,
                        VRADICFECP,
                        NRORG
                      )
                      VALUES
                      (
                        :P_CDFILIMPOIT,
                        :P_NRLANCIMPOIT,
                        :P_NRSEQIMPOIT,
                        :P_CDIMPOSTOIT,
                        :P_NRSEQITPRO,
                        :P_VRALIQIMPOIT,
                        0,
                        :P_VRREDBCIMPOIT,
                        :P_VRBCALCIMPOIT,
                        0,
                        :P_VRIMPOSTO,
                        :P_IDVALORINFO,
                        :P_VRADICALIQFECP,
                        :P_VRADICFECP,
                        :P_NRORG
                      )";
        
        $params = array(
            "P_CDFILIMPOIT"     =>  $cdfilimpoit,
            "P_NRLANCIMPOIT"    =>  $nrlancimpoit,
            "P_NRSEQIMPOIT"     =>  $nrseqimpoit,
            "P_CDIMPOSTOIT"     =>  $cdimpostoit,
            "P_NRSEQITPRO"      =>  $nrseqitpro,
            "P_VRALIQIMPOIT"    =>  $vraliqimpoit,
            "P_VRREDBCIMPOIT"   =>  $vrredbcimpoit,
            "P_VRBCALCIMPOIT"   =>  $vrbcalcimpoit,
            "P_VRIMPOSTO"       =>  $vrimposto,
            "P_IDVALORINFO"     =>  $idvalorinfo,
            "P_VRADICALIQFECP"  =>  $vradicaliqfecp,
            "P_VRADICFECP"      =>  $vradicfecp,
            "P_NRORG"           => $this->environment->getNrOrgTrab()
        );
        
      	$std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
	}
	
	public function deleteAliqTemp($cdfilial, $nrlanctonf, $cdimpotemp) {
	 
	    $sql = "DELETE
                FROM ALIQTEMPPROD
                WHERE NRALIQTEMP = :P_CDFILIAL
                  ||:P_NRLANCTONF
                AND CDIMPOTEMP = :P_CDIMPOTEMP";
	                    
        $params = array("P_CDFILIAL"	  => $cdfilial,
		                "P_NRLANCTONF"    => $nrlanctonf,
		                "P_CDIMPOTEMP"    => $cdimpotemp);   
		                
		$std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
	}
	
	public function deleteInciTemp($cdfilial, $nrlanctonf, $cdimpotemp) {
	 
	    $sql = "DELETE
                FROM INCITEMP
                WHERE NRINCITEMP = :P_CDFILIAL
                  ||:P_NRLANCTONF
                AND CDIMPOTEMP = :P_CDIMPOTEMP";
	                    
        $params = array("P_CDFILIAL"	  => $cdfilial,
		                "P_NRLANCTONF"    => $nrlanctonf,
		                "P_CDIMPOTEMP"    => $cdimpotemp);   
		                
		$std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
	}
	
	public function getSomaTrabitproimpos($cdfilial, $nrlanctonf) {
	    
	    $sql = "SELECT SUM(T.VRIMPOSIT) VRTOTAIMPO,
                  T.CDIMPOSTOIT
                FROM TRABITPROIMPOS T,
                  IMPOSTO I
                WHERE CDFILIMPOIT = :P_CDFILIAL
                AND NRLANCIMPOIT  = :P_NRLANCTONF
                AND IDPRODNOTA   <> 'C'
                AND T.CDIMPOSTOIT = I.CDIMPOSTO
                GROUP BY CDIMPOSTOIT
                ORDER BY CDIMPOSTOIT";
	    
	    $params = array("P_CDFILIAL"	  => $cdfilial,
		                "P_NRLANCTONF"    => $nrlanctonf);   
		                
		$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
	
	    return $std->fetchAll();
	}
	
	public function getValorTotalImpostoRetido($cdfilial, $nrlanctonf) {
	    $sql = "SELECT SUM(VRTOTIMPONF) VRTOTIMPONF
                FROM IMPOSNFTOT NF,
                  IMPOSTO I
                WHERE NF.CDFILIAL = :P_CDFILIAL
                AND NF.NRLANCTONF = :P_NRLANCTONF
                AND NF.CDIMPOSTO  = I.CDIMPOSTO
                AND I.IDTRATIMPO  = 'R'";
                
        $params = array("P_CDFILIAL"	  => $cdfilial,
		                "P_NRLANCTONF"    => $nrlanctonf);   
		  
		$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
	
	    return $std->fetch()["VRTOTIMPONF"];
	}
	
	public function getRateioAnterior($cdfilial, $nrlanctonf, $nrseqitprod) {
	    $sql = "SELECT CDFILIAL,
                  NRLANCTONF,
                  NRSEQITPROD,
                  CDFILICCUST,
                  CDCENTCUSNF,
                  VRRATENF,
                  NVL(VRPERRAT, 0) AS VRPERC
                FROM RATEIONF
                WHERE CDFILIAL  = :P_CDFILIAL
                AND NRLANCTONF  = :P_NRLANCTONF
                AND NRSEQITPROD = :P_NRSEQITPROD
                ORDER BY NRSEQITPROD";
                
        $params = array("P_CDFILIAL"	  => $cdfilial,
		                "P_NRLANCTONF"    => $nrlanctonf,
		                "P_NRSEQITPROD"   => $nrseqitprod);   
		                
		$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
	
	    return $std->fetchAll();
	}
	
	public function getValorRetidoPos($cdfilial, $nrlanctonf) {
	    $sql = "SELECT SUM(VRTOTIMPONF) VRTOTIMPONF
                FROM IMPOSNFTOT NF,
                  IMPOSTO I
                WHERE NF.CDFILIAL = :P_CDFILIAL
                AND NF.NRLANCTONF = :P_NRLANCTONF
                AND NF.CDIMPOSTO  = I.CDIMPOSTO
                AND I.IDTRATIMPO  = 'R'";
                
        $params = array("P_CDFILIAL"	  => $cdfilial,
		                "P_NRLANCTONF"    => $nrlanctonf);   
		                
		$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
	  
	    return (float) $std->fetch()["VRTOTIMPONF"];
	    
	}
	
	public function consisteParametrizacaoFiscal($cdtipooper, $identrsaid) {
	    $sql = "SELECT NVL(IDCONSPARFIS, 'S') IDCONSPARFIS,
                  IDCONTSOMENIMPO
                FROM TIPOOPERAC
                WHERE CDTIPOOPER = :P_CDTIPOOPER
                AND IDENTRSAIDOP = :P_IDENTRSAID";
                
        $params = array("P_CDTIPOOPER"	    => $cdtipooper,
		                 "P_IDENTRSAID"     => $identrsaid);   
                
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        
		$consiste = $std->fetch();
		
	    return $consiste["IDCONSPARFIS"] == 'S';
	}
	
	public function getCstImpostos($cdfilial, $nrlanctonf) {
	    $sql = "SELECT NVL(II.CDSITUTRIBIT, 'XXX') CDSITUTRIBIT,
	              NVL(CDFISCOPERIT, 'XXX') CDFISCOPERIT,
                  II.CDIMPOSTO,
                  P.CDARVPROD,
                  P.NMPRODUTO
                FROM ITEMPROIMPOS II,
                  ITEMPRODUTO IP,
                  PRODUTO P
                WHERE II.CDFILIAL  = :P_CDFILIAL
                AND II.NRLANCTONF  = :P_NRLANCTONF
                AND II.CDFILIAL    = IP.CDFILIAL
                AND II.NRLANCTONF  = IP.NRLANCTONF
                AND II.NRSEQITPROD = IP.NRSEQITPROD
                AND IP.CDPRODUTO   = P.CDPRODUTO";
                
        $params = array("P_CDFILIAL"	  => $cdfilial,
		                "P_NRLANCTONF"    => $nrlanctonf);   
                
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        
		return $std->fetchAll();
	}
	
	public function getCstItens($cdfilial, $nrlanctonf) {
	    
	    $sql = "SELECT IP.CDSITUTRIBPR,
                  IP.CDFISOPEITEM,
                  P.CDARVPROD
                FROM ITEMPRODUTO IP,
                  PRODUTO P
                WHERE IP.CDFILIAL     = :P_CDFILIAL
                AND IP.NRLANCTONF     = :P_NRLANCTONF
                AND P.CDPRODUTO       = IP.CDPRODUTO
                AND (IP.CDSITUTRIBPR IS NULL
                OR IP.CDFISOPEITEM   IS NULL)";
                
        $params = array("P_CDFILIAL"	  => $cdfilial,
		                "P_NRLANCTONF"    => $nrlanctonf);   
                
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        
		return $std->fetchAll();
	    
	}
	
	public function getItemProImpos($cdfilial, $nrlanctonf, $nrseqitprod) {
	    $sql = "SELECT CDIMPOSTO
                FROM ITEMPROIMPOS
                WHERE CDFILIAL  = :P_CDFILIAL
                AND NRLANCTONF  = :P_NRLANCTONF
                AND NRSEQITPROD = :P_NRSEQITPROD";
                
        $params = array("P_CDFILIAL"	  => $cdfilial,
		                "P_NRLANCTONF"    => $nrlanctonf,
		                "P_NRSEQITPROD"   => $nrseqitprod);   
                
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        
		return $std->fetchAll();
	}
	
	public function getMapasBloqueados($cdfilial, $nrlanctonf) {
	    $sql = "SELECT I.NRSEQUMAPAPU, I.NRSEQMAPOIMP, M.CDIMPOSTO, III.SGIMPOSTONFE
                FROM IMPOSTONF I, MAPAAPURAC M, IMPOSTO II, IMPOSTONFE III
                WHERE I.CDFILIAL = :P_FILIAL
                AND I.NRLANCTONF = :P_LANCTONF
                AND ((I.NRSEQUMAPAPU = M.NRSEQUMAPAPU) OR (M.NRSEQUMAPAPU = I.NRSEQMAPOIMP))
                AND II.CDIMPOSTONFE = M.CDIMPOSTO
                AND III.CDIMPOSTONFE = II.CDIMPOSTONFE
                AND M.IDBLOQMAPAPU = 'S'
                AND II.CDIMPOSTO = I.CDIMPOSTO
                GROUP BY I.NRSEQUMAPAPU, I.NRSEQMAPOIMP, M.CDIMPOSTO, III.SGIMPOSTONFE
                ORDER BY CDIMPOSTO";
                
        $params = array("P_FILIAL"	  => $cdfilial,
                        "P_LANCTONF"    => $nrlanctonf);   
                
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        
		return $std->fetchAll();
	}
	
	public function delMapa($cdfilial, $nrlanctonf){
	    $sql = "DELETE FROM ITMAPARECUP
                WHERE CDFILIAL   = :P_FILIAL
                AND NRLANCTONF = :P_LANCTONF";
                
        $params = array("P_FILIAL"	  => $cdfilial,
                        "P_LANCTONF"    => $nrlanctonf);   
                
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
                
	}
	
	public function getVenctosNotaEnt($cdfilial, $nrlanctonf, $nrorg) {
	    $sql = "SELECT CDFILIAL, NRLANCTONF, CDTPCTPAGAR, NRSEQVENPAG, NRPARCELAPAG,
	                   QTDIASVENPAG, DTORIGVENPAG, VRORIGPAG,
	                   DTATUAVENPAG, VRATUAPAG, IDSTATPAG, CDFILIBPAG, NRLANBOLPAG
	              FROM CONTAPAGAR
	              WHERE (CDFILIAL   = :P_CDFILIAL)
	                AND (NRLANCTONF = :P_NRLANCTONF)
	                AND (NRORG      = :P_NRORG)
	           ORDER BY 1, 2, 5";
                
        $params = array("P_CDFILIAL"    => $cdfilial,
                        "P_NRLANCTONF"  => $nrlanctonf,
                        "P_NRORG"       => $nrorg);
	    
	    $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        
		return $std->fetchAll();
	}
	
	public function getTipoContaPagarForn($cdfilial, $nrlanctonf, $nrorg) {
	     $sql = "SELECT MAX (F.CDTPCTPAGAR) AS CDTPCTPAGAR
                   FROM NOTAFISCAL N, FORNECEDOR F
                  WHERE N.CDFILIAL   = :P_CDFILIAL
                    AND N.NRLANCTONF = :P_NRLANCTONF
                    AND N.NRORG      = :P_NRORG
                    AND F.CDFORNECED = N.CDFORNECED
                    AND N.NRORG = F.NRORG";
                
        $params = array("P_CDFILIAL"    => $cdfilial,
                        "P_NRLANCTONF"  => $nrlanctonf,
                        "P_NRORG"       => $nrorg);
	    
	    $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        
		return $std->fetch();
	}
	
	public function getTipooperacCta($identrsaidop, $cdtipooper, $nrorg) {
	    $sql = "SELECT T.CDTPCTPAGAR,
                       T.CDTPCTRECEB
                  FROM TIPOOPERAC T
                 WHERE T.IDENTRSAIDOP = :P_IDENTRSAIDOP
                   AND T.CDTIPOOPER   = :P_CDTIPOOPER
                   AND T.NRORG        = :P_NRORG";
        
        $params = array("P_IDENTRSAIDOP"   => $identrsaidop,
                        "P_CDTIPOOPER"     => $cdtipooper,
                        "P_NRORG"          => $nrorg);
        
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        
        return $std->fetch();
	}
	
	public function getTipoContaReceberProd($cdfilial, $nrlanctonf, $nrorg) {
	    $sql = "SELECT P3.CDPRODUTO, P3.CDARVPROD, P4.CDARVPROD AS CDNIVEL, P4.CDTPCTRECEB,
                       ROUND(A.VRBRUTITEM,2) AS VRBRUTITEM
                   FROM ITEMPRODUTO A, PRODUTO P3, PRODUTO P4
                  WHERE A.CDFILIAL = :P_CDFILIAL
                    AND A.NRLANCTONF = :P_NRLANCTONF
                    AND A.NRORG = :P_NRORG
                    AND A.CDPRODUTO = P3.CDPRODUTO
                    AND A.NRORG = P3.NRORG
                    AND (INSTR(P3.CDARVPROD,NVL(RTRIM(P4.CDARVPROD),P3.CDARVPROD)) = 1)
                    AND P4.CDTPCTRECEB IS NOT NULL
                    AND P4.CDARVPROD = (SELECT MAX(P1.CDARVPROD)
                                          FROM ITEMPRODUTO IT,  PRODUTO P, PRODUTO P1
                                         WHERE IT.CDFILIAL = A.CDFILIAL
                                           AND IT.NRLANCTONF = A.NRLANCTONF
                                           AND IT.CDPRODUTO = A.CDPRODUTO
                                           AND IT.NRORG = A.NRORG
                                           AND IT.CDPRODUTO = P.CDPRODUTO
                                           AND IT.NRORG = P.NRORG
                                           AND (INSTR(P.CDARVPROD,NVL(RTRIM(P1.CDARVPROD),P.CDARVPROD)) = 1)
                                           AND P1.CDTPCTRECEB IS NOT NULL)
                    ORDER BY P3.CDARVPROD";
	    
	    $params = array("P_CDFILIAL"    => $cdfilial,
                        "P_NRLANCTONF"  => $nrlanctonf,
                        "P_NRORG"       => $nrorg);
	    
	    $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        
		return $std->fetchAll();
	}
	
	public function getVenctosNotaSaida($cdfilial, $nrlanctonf, $nrorg) {
	    $sql = "SELECT CDFILIAL, NRLANCTONF, CDTPCTRECEB, NRSEQVENREC, NRPARCELAREC,
                       QTDIASVENREC, DTORIGVENREC, VRORIGREC,      
                       DTATUAVENREC, VRATUAREC, IDSTATREC, CDFILIBOLREC, NRLANBOLREC
                  FROM CONTARECEBER
                 WHERE (CDFILIAL   = :P_CDFILIAL)
                   AND (NRLANCTONF = :P_NRLANCTONF)
                   AND (NRORG      = :P_NRORG)
                 ORDER BY 1, 2, 5";
                
        $params = array("P_CDFILIAL"    => $cdfilial,
                        "P_NRLANCTONF"  => $nrlanctonf,
                        "P_NRORG"       => $nrorg);
	    
	    $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        
		return $std->fetchAll();
	}
	
	public function updateDtReplicatedNf($dtEmissao, $nrnotafisc, $chaveAcesso, $iddocreplic){
	    $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        
	    $query = "UPDATE NOTAFISCAL
                    SET DTEMISSAO   = :P_DTEMISSAO,
                        NRNOTAFISC  = :P_NRNOTAFISC,
                        NRACESSONFE = :P_NRACESSONFE
                  WHERE IDDOCREPLIC           = :P_IDDOCREPLIC AND
                          CDFILIAL||NRLANCTONF <> :P_IDDOCREPLIC AND
                          IDENTRSAIDOP = 'E'";
                
        $params = array("P_DTEMISSAO"    => $dtEmissao,
                        "P_NRNOTAFISC"  => $nrnotafisc,
                        "P_NRACESSONFE"       => $chaveAcesso,
                        "P_IDDOCREPLIC" => $iddocreplic);
	    
    	$std = $connection->prepare($query);
        $std->execute($params);
        $connection->commit();
	}
	
	public function updateReplicatedNfOL($dtEmissao, $nrnotafisc, $chaveAcesso, $dsnotafisorigol){
	    $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
	    $query = "UPDATE NOTAFISCAL
                       SET NRNOTAFISC  = :P_NRNOTAFISC,
                           DTEMISSAO   = :P_DTEMISSAO,
                           NRACESSONFE = :P_NRACESSONFE
                     WHERE DSNOTAFISORIGOL = :DSNOTAFISORIGOL";
                
        $params = array("P_DTEMISSAO"    => $dtEmissao,
                        "P_NRNOTAFISC"  => $nrnotafisc,
                        "P_NRACESSONFE"       => $chaveAcesso,
                        "DSNOTAFISORIGOL" => $dsnotafisorigol);
	    
	    
    	
    	$std = $connection->prepare($query);
        $std->execute($params);
        $connection->commit();
	}
	
	public function updateNfDevoOL($nrlanctonf_transmitida, $cdfilial_transmitida){
	    $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        
	    $query_getNfs = "SELECT I.CDFILIAL, I.NRLANCTONF, N.NRNOTAFISC
                  FROM ITEMNOTAFISC I, NOTAFISCAL N
                 WHERE I.CDFILIALIT   = :P_FILIAL
                   AND I.NRLANCTONFIT = :P_LANCAMENTO
                   AND I.CDFILIAL   = N.CDFILIAL
                   AND I.NRLANCTONF = N.NRLANCTONF
                   AND N.IDNFDEVOOL = 'S'";
                   
        $params_getNfs = array("P_FILIAL"    => $cdfilial_transmitida,
                        "P_LANCAMENTO"  => $nrlanctonf_transmitida);
	              
        
    	$db_connection_query_getNfs = $connection->prepare($query_getNfs);
        $db_connection_query_getNfs->execute($params_getNfs);
        $OLs = $db_connection_query_getNfs->fetchAll();
        
        if(!empty($OLs)){
            foreach($OLs as $ol){
                
                $query_ol = "UPDATE NOTAFISCAL
                             SET NRNOTAFISC = :NRNOTAFISC
                           WHERE CDFILIAL = :P_CDFILIAL
                             AND NRLANCTONF = :P_NRLANCTONF";
                             
                $ol_cdFilial = $ol['CDFILIAL'];
                $ol_nrLanctoNf = $ol['NRLANCTONF'];
                $ol_nrNotaFisc = $ol['NRNOTAFISC'];
                
                $params_ol = array("P_CDFILIAL"    => $ol_cdFilial,
                                   "P_NRLANCTONF"  => $ol_nrLanctoNf,
                                   "NRNOTAFISC"    => $ol_nrNotaFisc);
                
                
            	$db_connection_query_ol = $connection->prepare($query);
                $db_connection_query_ol->execute($params);
            }
        }
        $connection->commit();
	}
}
