<?php

namespace Teknisa\DF\Factories;
use Doctrine\ORM\EntityManager;

class TipooperacFilialFactory {
    protected $entityManager;
    protected $environment;
    
    public function __construct(EntityManager $entityManager, $environment) {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
    }
    
    public function put_filial($cdfilial, $cdtipooper, $nrorg, $idliberado, $idlancmanual){
        $sql = "UPDATE TIPOOPERACFIL 
                    SET IDLIBERADO = :IDLIBERADO,
                        IDLANCMANUAL = :IDLANCMANUAL,
                        DTULTATU  = SYSDATE
                    WHERE CDFILIAL = :CDFILIAL
                        AND CDTIPOOPER = :CDTIPOOPER
                        AND NRORG = :NRORG 
                        AND IDENTRSAIDOP = 'S'";
              
        $params = array(
                         "IDLIBERADO"    => $idliberado,
                         "IDLANCMANUAL"  => $idlancmanual,
                         "CDFILIAL" => $cdfilial,
                         "CDTIPOOPER" => $cdtipooper,
                         "NRORG" => $nrorg
                        );
               
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        
        $std = $connection->prepare($sql);
        $std->execute($params);
        
        $connection->commit();
    }
    
    public function insert_filial($cdfilial, $cdtipooper, $idliberado, $idlancmanual){
        $sql = "
                  INSERT INTO TIPOOPERACFIL(CDFILIAL, IDENTRSAIDOP, CDTIPOOPER, IDLIBERADO, IDLANCMANUAL, DTULTATU, NRORG)
                                    VALUES (:CDFILIAL,'S', :CDTIPOOPER, :IDLIBERADO, :IDLANCMANUAL, SYSDATE, :NRORG)
               ";
              
        $params = array(
                         "IDLIBERADO"    => $idliberado,
                         "IDLANCMANUAL"  => $idlancmanual,
                         "CDFILIAL" => $cdfilial,
                         "CDTIPOOPER" => $cdtipooper,
                         "NRORG" => $this->environment->getNrOrgTrab()
                        );
               
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        
        $std = $connection->prepare($sql);
        $std->execute($params);
        
        $connection->commit();
    }
    
    public function insert_cliente( $cdfilial, $cdcliente, $cdtipooper){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        
        $sql_max = "SELECT NVL(MAX(NRSEQTPOPER), 0) + 1 AS SEQUENCIAL FROM TIPOOPERACCLIE";
        $std_max = $connection->prepare($sql_max);
        $std_max->execute();
        $nrseqtpoper = $std_max->fetch()["SEQUENCIAL"];
        
        $sql = "INSERT INTO TIPOOPERACCLIE(NRSEQTPOPER, CDFILIAL, IDENTRSAIDOP, CDTIPOOPER, CDCLIENTE)
                    VALUES(:NRSEQTPOPER, :CDFILIAL, :IDENTRSAIDOP, :CDTIPOOPER, :CDCLIENTE)";
              
        $params = array(
                         "NRSEQTPOPER"  => strval($nrseqtpoper),
                         "CDFILIAL"     => $cdfilial,
                         "IDENTRSAIDOP" => "S",
                         "CDCLIENTE"    => $cdcliente,
                         "CDTIPOOPER"   => $cdtipooper
                        );
               
        
        
        
        $std = $connection->prepare($sql);
        $std->execute($params);
        
        $connection->commit();
        
    }
    
    public function cliente_delete($cdfilial, $cdtipooper, $cdcliente, $identrsaidop, $sequencial){
        $query = "DELETE
                      FROM TIPOOPERACCLIE
                     WHERE CDFILIAL     = :CDFILIAL
                       AND CDTIPOOPER   = :CDTIPOOPER
                       AND CDCLIENTE    = :CDCLIENTE
                       AND IDENTRSAIDOP = :IDENTRSAIDOP
                       AND NRSEQTPOPER  = :NRSEQTPOPER";
        $parametros_query = array(
            "CDFILIAL" => $cdfilial,
            "CDTIPOOPER" => $cdtipooper,
            "CDCLIENTE" => $cdcliente,
            "IDENTRSAIDOP" => $identrsaidop,
            "NRSEQTPOPER" => $sequencial,
        );
        
        $conexao_sql = $this->entityManager->getConnection()->prepare($query);
        $conexao_sql->execute($parametros_query);
    }
    
    public function endereco_get($cdfilial){
        $sql = "SELECT 'C' AS IDTPENDEFILI, F.NMFILIAL, F.NMRAZSOCFILI, F.NRINSJURFILI,
                       F.CDINSCESTA, F.CDINSCMUNI, E.DSENDEFILI, E.DSCOMPENDFIL,
                       E.CDPAIS, P.NMPAIS, E.SGESTADO, E.CDMUNICIPIO,
                       M.NMMUNICIPIO, E.NMBAIRFILI, E.CDBAIRRO, B.NMBAIRRO,
                       E.NRCEPFILI, E.NRTELEFILI, CASE WHEN E.CDFILIAL IS NULL THEN 'NAO_CADASTRADO' ELSE 'CADASTRADO' END AS EXISTEINFO
                  FROM FILIAL F LEFT JOIN ENDEFILI E ON (F.CDFILIAL = E.CDFILIAL)
                                                    AND (E.IDTPENDEFILI = 'C')
                                LEFT JOIN PAIS P ON (E.CDPAIS = P.CDPAIS)
                                LEFT JOIN MUNICIPIO M ON (E.CDPAIS = M.CDPAIS)
                                                     AND (E.SGESTADO = M.SGESTADO)
                                                     AND (E.CDMUNICIPIO = M.CDMUNICIPIO)
                                LEFT JOIN BAIRRO B ON (E.CDPAIS = B.CDPAIS)
                                                  AND (E.SGESTADO = B.SGESTADO)
                                                  AND (E.CDMUNICIPIO = B.CDMUNICIPIO)
                                                  AND (E.CDBAIRRO = B.CDBAIRRO)
                 WHERE F.CDFILIAL = :P_CDFILIAL
                 UNION ALL
                SELECT 'E' AS IDTPENDEFILI, F.NMFILIAL, F.NMRAZSOCFILI, F.NRINSJURFILI,
                       F.CDINSCESTA, F.CDINSCMUNI, E.DSENDEFILI, E.DSCOMPENDFIL,
                       E.CDPAIS, P.NMPAIS, E.SGESTADO, E.CDMUNICIPIO,
                       M.NMMUNICIPIO, E.NMBAIRFILI, E.CDBAIRRO, B.NMBAIRRO,
                       E.NRCEPFILI, E.NRTELEFILI, CASE WHEN E.CDFILIAL IS NULL THEN 'NAO_CADASTRADO' ELSE 'CADASTRADO' END AS EXISTEINFO
                  FROM FILIAL F LEFT JOIN ENDEFILI E ON (F.CDFILIAL = E.CDFILIAL)
                                                    AND (E.IDTPENDEFILI = 'E')
                                LEFT JOIN PAIS P ON (E.CDPAIS = P.CDPAIS)
                                LEFT JOIN MUNICIPIO M ON (E.CDPAIS = M.CDPAIS)
                                                     AND (E.SGESTADO = M.SGESTADO)
                                                     AND (E.CDMUNICIPIO = M.CDMUNICIPIO)
                                LEFT JOIN BAIRRO B ON (E.CDPAIS = B.CDPAIS)
                                                  AND (E.SGESTADO = B.SGESTADO)
                                                  AND (E.CDMUNICIPIO = B.CDMUNICIPIO)
                                                  AND (E.CDBAIRRO = B.CDBAIRRO)
                 WHERE F.CDFILIAL = :P_CDFILIAL
                 UNION ALL
                SELECT 'I' AS IDTPENDEFILI, F.NMFILIAL, F.NMRAZSOCFILI, F.NRINSJURFILI,
                       F.CDINSCESTA, F.CDINSCMUNI, I.DSENDEIEST AS DSENDEFILI, I.DSCOMPENDIES AS DSCOMPENDFIL,
                       I.CDPAIS, P.NMPAIS, I.SGESTADO, I.CDMUNICIPIO,
                       M.NMMUNICIPIO, I.NMBAIRIEST AS NMBAIRFILI, I.CDBAIRRO, B.NMBAIRRO,
                       I.NRCEPIEST AS NRCEPFILI, I.NRTELEIEST AS NRTELEFILI, CASE WHEN I.CDINSCESTA IS NULL THEN 'NAO_CADASTRADO' ELSE 'CADASTRADO' END AS EXISTEINFO
                  FROM FILIAL F LEFT JOIN INSCRESTAD I ON (F.CDEMPRESA = I.CDEMPRESA)
                                                      AND (F.CDINSCESTA = I.CDINSCESTA)
                                LEFT JOIN PAIS P ON (I.CDPAIS = P.CDPAIS)
                                LEFT JOIN MUNICIPIO M ON (I.CDPAIS = M.CDPAIS)
                                                     AND (I.SGESTADO = M.SGESTADO)
                                                     AND (I.CDMUNICIPIO = M.CDMUNICIPIO)
                                LEFT JOIN BAIRRO B ON (I.CDPAIS = B.CDPAIS)
                                                  AND (I.SGESTADO = B.SGESTADO)
                                                  AND (I.CDMUNICIPIO = B.CDMUNICIPIO)
                                                  AND (I.CDBAIRRO = B.CDBAIRRO)
                 WHERE F.CDFILIAL = :P_CDFILIAL
                 UNION ALL
                SELECT 'P' AS IDTPENDEFILI, F.NMFILIAL, F.NMRAZSOCFILI, F.NRINSJURFILI,
                       F.CDINSCESTA, F.CDINSCMUNI, E.DSENDEFILI, E.DSCOMPENDFIL,
                       E.CDPAIS, P.NMPAIS, E.SGESTADO, E.CDMUNICIPIO,
                       M.NMMUNICIPIO, E.NMBAIRFILI, E.CDBAIRRO, B.NMBAIRRO,
                       E.NRCEPFILI, E.NRTELEFILI, CASE WHEN E.CDFILIAL IS NULL THEN 'NAO_CADASTRADO' ELSE 'CADASTRADO' END AS EXISTEINFO
                  FROM FILIAL F LEFT JOIN ENDEFILI E ON (F.CDFILIAL = E.CDFILIAL)
                                                    AND (E.IDTPENDEFILI = 'P')
                                LEFT JOIN PAIS P ON (E.CDPAIS = P.CDPAIS)
                                LEFT JOIN MUNICIPIO M ON (E.CDPAIS = M.CDPAIS)
                                                     AND (E.SGESTADO = M.SGESTADO)
                                                     AND (E.CDMUNICIPIO = M.CDMUNICIPIO)
                                LEFT JOIN BAIRRO B ON (E.CDPAIS = B.CDPAIS)
                                                  AND (E.SGESTADO = B.SGESTADO)
                                                  AND (E.CDMUNICIPIO = B.CDMUNICIPIO)
                                                  AND (E.CDBAIRRO = B.CDBAIRRO)
                 WHERE F.CDFILIAL = :P_CDFILIAL
                 UNION ALL
                SELECT 'T' AS IDTPENDEFILI, F.NMFILIAL, F.NMRAZSOCFILI, F.NRINSJURFILI,
                       F.CDINSCESTA, F.CDINSCMUNI, E.DSENDEFILI, E.DSCOMPENDFIL,
                       E.CDPAIS, P.NMPAIS, E.SGESTADO, E.CDMUNICIPIO,
                       M.NMMUNICIPIO, E.NMBAIRFILI, E.CDBAIRRO, B.NMBAIRRO,
                       E.NRCEPFILI, E.NRTELEFILI, CASE WHEN E.CDFILIAL IS NULL THEN 'NAO_CADASTRADO' ELSE 'CADASTRADO' END AS EXISTEINFO
                  FROM FILIAL F LEFT JOIN ENDEFILI E ON (F.CDFILIAL = E.CDFILIAL)
                                                    AND (E.IDTPENDEFILI = 'T')
                                LEFT JOIN PAIS P ON (E.CDPAIS = P.CDPAIS)
                                LEFT JOIN MUNICIPIO M ON (E.CDPAIS = M.CDPAIS)
                                                     AND (E.SGESTADO = M.SGESTADO)
                                                     AND (E.CDMUNICIPIO = M.CDMUNICIPIO)
                                LEFT JOIN BAIRRO B ON (E.CDPAIS = B.CDPAIS)
                                                  AND (E.SGESTADO = B.SGESTADO)
                                                  AND (E.CDMUNICIPIO = B.CDMUNICIPIO)
                                                  AND (E.CDBAIRRO = B.CDBAIRRO)
                 WHERE F.CDFILIAL = :P_CDFILIAL
                 UNION ALL
                SELECT 'X' AS IDTPENDEFILI, F.NMFILIAL, F.NMRAZSOCFILI, F.NRINSJURFILI,
                       F.CDINSCESTA, F.CDINSCMUNI, E.DSENDEFILI, E.DSCOMPENDFIL,
                       E.CDPAIS, P.NMPAIS, E.SGESTADO, E.CDMUNICIPIO,
                       M.NMMUNICIPIO, E.NMBAIRFILI, E.CDBAIRRO, B.NMBAIRRO,
                       E.NRCEPFILI, E.NRTELEFILI, CASE WHEN E.CDFILIAL IS NULL THEN 'NAO_CADASTRADO' ELSE 'CADASTRADO' END AS EXISTEINFO
                  FROM FILIAL F LEFT JOIN ENDEFILI E ON (F.CDFILIAL = E.CDFILIAL)
                                                    AND (E.IDTPENDEFILI = 'X')
                                LEFT JOIN PAIS P ON (E.CDPAIS = P.CDPAIS)
                                LEFT JOIN MUNICIPIO M ON (E.CDPAIS = M.CDPAIS)
                                                     AND (E.SGESTADO = M.SGESTADO)
                                                     AND (E.CDMUNICIPIO = M.CDMUNICIPIO)
                                LEFT JOIN BAIRRO B ON (E.CDPAIS = B.CDPAIS)
                                                  AND (E.SGESTADO = B.SGESTADO)
                                                  AND (E.CDMUNICIPIO = B.CDMUNICIPIO)
                                                  AND (E.CDBAIRRO = B.CDBAIRRO)
                 WHERE F.CDFILIAL = :P_CDFILIAL";
              
        $params = array(
                         "P_CDFILIAL" => $cdfilial
                        );
               
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        
        $std = $connection->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }
    
    public function endereco_put($cdfilial, $cdtipooper, $idendemistpopfil){
        $sql = "UPDATE TIPOOPERACFIL
                SET IDENDEMISTPOPFIL = :IDENDEMISTPOPFIL
                    DTULTATU  = SYSDATE
                WHERE CDFILIAL    = :CDFILIAL
                   AND IDENTRSAIDOP = 'S'
                   AND CDTIPOOPER   = :CDTIPOOPER";
              
        $params = array(
                         "IDENDEMISTPOPFIL"  => $idendemistpopfil,
                         "CDFILIAL" => $cdfilial,
                         "CDTIPOOPER" => $cdtipooper
                        );
               
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        
        $std = $connection->prepare($sql);
        $std->execute($params);
        
        $connection->commit();
    }
    
}