<?php

namespace Teknisa\DF\Modules\Entrada\Controllers;

use Teknisa\DF\Modules\Entrada\Services\Dfe11000NfEntradaService;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource;
use \Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;

class Dfe11000NfEntradaController extends \Teknisa\DF\Controllers\TeknisaCrud{
    protected $nfEntradaService;
    protected $validacoesEstoqueService;
    protected $fileServerBaseUrl;
    protected $lancamentoNotafiscalService;
    protected $erros = array();
    protected $success = array();
    protected $act = array();
    
    public function __construct($crudService, $parameterBag, Dfe11000NfEntradaService $nfEntradaService, $validacoeEstoqueService, $lancamentoNotafiscalService, $reportService, $entityManager, $environment, $fileServerBaseUrl) {
        $this->nfEntradaService = $nfEntradaService;
        $this->validacoesEstoqueService = $validacoeEstoqueService;
        $this->reportService = $reportService;
        $this->crudService   = $crudService;
        $this->entityManager = $entityManager;
        $this->environment   = $environment;
        $this->parameterBag  = $parameterBag;
        $this->fileServerBaseUrl = $fileServerBaseUrl;
        $this->lancamentoNotafiscalService = $lancamentoNotafiscalService;
    }
    
    public function verificaRateio(Request\Row $request, Response $response){
        $row = $request->getRow();
        $dadosRateio = $this->nfEntradaService->verificaRateio($row['CDFILIAL'], $row['NRLANCTONF']);
        $response->addDataset(new DataSource\DataSet('dadosRateio', $dadosRateio));
        
    }

    public function excluirNotaEntrada(Request\Row $request, Response $response) {
        try {
            $rowLancamento = $request->getRow();

            $filial = $rowLancamento['CDFILIAL'];
            $lancamento = $rowLancamento['NRLANCTONF'];

            if($this->nfEntradaService->verificaNotaSendoUtilizada($filial, $lancamento)) {
                throw new \Exception('Operação bloqueada. A nota fiscal está sendo utilizada por outro operador!');
            }
            
            if($this->nfEntradaService->verificaAuditoria($filial, $lancamento)) {
                throw new \Exception('Existe análise/auditoria para a nota fiscal selecionada');
            }
            
            $this->nfEntradaService->setNotafiscalEntity($filial, $lancamento);
            $this->nfEntradaService->deleteNotaFiscalEntrada();

            $response->addDataset(new DataSource\DataSet('excluirNotaEntrada', array('success' => 'S')));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function intgItemNotaFisc(Request\Row $request, Response $response) {
        $row = $request->getRow();
        //var_dump($row);
        $referenciados = $this->nfEntradaService->intgItemNotaFisc($row['NRACESSONFE']);
        $response->addDataset(new DataSource\DataSet('intg_notafiscal', array($referenciados)));
    }

    public function realizaImportacaoAf(Request\Row $request, Response $response) {
        try {
            $rowLancamento = $request->getRow();

            if (\Zeedhi\Framework\Util\Functions::arrayKeyExists('AF', $rowLancamento)) {
                $afSelecionadas = $rowLancamento['AF'];

                $this->nfEntradaService->setNotafiscalEntity($rowLancamento['CDFILIAL'], $rowLancamento['NRLANCTONF']);
                $this->nfEntradaService->realizaImportacaoAf($rowLancamento, $afSelecionadas);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function salvaImpostosNotaFiscal(Request\DataSet $request, Response $response) {
        try {
            $rows = $request->getDataSet()->getRows();

            if (is_array($rows))
                $this->nfEntradaService->salvaImpostosNotaFiscal($rows);

        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function atualizaDadosProduto(Request\Row $request, Response $response) {
        $rowRelacionamento = $request->getRow();
        
        
        $connection = $this->entityManager->getConnection();
        
        try {
            
            $connection->beginTransaction();
            $cdfilial = $rowRelacionamento[0]['CDFILIAL'];
            $nrlanctonf = $rowRelacionamento[0]['NRLANCTONF'];
            // Alteração para pegar os itens antes de excluir para salvar dados da AF ao inserir os itens novamente
            $nf = array("CDFILIAL"=>$cdfilial,
                        "NRLANCTONF"=>$nrlanctonf                   );
            $rowItensAnteriores = $this ->nfEntradaService ->getItensNfEntrada($nf);
            // Alteração para pegar os itens antes de excluir para salvar dados da AF ao inserir os itens novamente
            $this->nfEntradaService->deleteAuditoria($cdfilial, $nrlanctonf);
            $this->nfEntradaService->deleteProdutos($cdfilial, $nrlanctonf,$rowRelacionamento);
            $this->nfEntradaService->atualizaDadosProduto($cdfilial, $nrlanctonf,$rowRelacionamento, $rowItensAnteriores);
            
            $connection->commit();
                
        }
        catch(\Exception $e) {
            $connection->rollback();
            $response->setError(new Error($e->getMessage(), 400));
            $response->setStatus('400');
        }
    }
    
    public function validaAF(Request\Row $request, Response $response){
        try {
            $row = $request->getRow();

            $cdfilial = $row['CDFILIRECE'];
            $cdtipooper = $row['CDTIPOOPER'];
            $cdforneced = $row['CDFORNECED'];

            $obrigaAF = $this->nfEntradaService->validaAFSalvarNota($cdfilial, $cdtipooper, $cdforneced);
            $response->addDataset(new DataSource\DataSet('validaAF', array($obrigaAF)));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function removeTrabsLancamentoEntrada(Request\Row $request, Response $response) {
        try {
            $rowLancamento = $request->getRow();

            $filial = $rowLancamento['CDFILIAL'];
            $lancamento = $rowLancamento['NRLANCTONF'];

            $this->nfEntradaService->removeTrabsLancamentoEntrada($filial, $lancamento);

        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function validaInscEstadual(Request\Row $request, Response $response) {
        try{
            $this->nfEntradaService->validaInscEstadual($request->getRow(), $response);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /*
    *  Salva o cabeçalho da Nota Fiscal de Entrada e realiza as operações necessárias
    *  para a nota
    **/
    
    public function salvarNotaEntrada(Request\Row $request, Response $response) {
        $rowLancamento = $request->getRow();
        
        $connection = $this->entityManager->getConnection();
        
        $isApp  = isset($rowLancamento["APP"]) ? $rowLancamento["APP"] : false;
        $commit = false;

        $verificou = false;

        if(isset($rowLancamento["NRLANCTONF"])) {
            if($this->nfEntradaService->verificaNotaSendoUtilizada($rowLancamento["CDFILIAL"], $rowLancamento["NRLANCTONF"])) {
                throw new \Exception('Operação bloqueada. A nota fiscal está sendo utilizada por outro operador!');
            }
            else{
                $verificou = true;
            }
        }
        
        try {
            
            $connection->beginTransaction();
             
            $rowLancamento = $request->getRow();
            
            $nracessonfe = $rowLancamento["NRACESSONFE"];
            $sequencial = $this->nfEntradaService->getSequencialHistoricoXML($rowLancamento["NRACESSONFE"])[0]["NRSEQHISTOR"];
            if(isset($rowLancamento["AF"])) {
                if(!empty($rowLancamento["AF"])) {
                    $temAf = true;
                }
                else {
                    $temAf = false;
                }
            }
            else {
                $temAf = false;
            }
            if($rowLancamento["NRACESSONFE"]) {
                // Verifica se os itens possuem os relacionamentos necessarios
                // para o lançamento
                $this->nfEntradaService->validacoesItens($rowLancamento);
            }
            
            // Faz as validações necessarias para salvar o cabeçalho
            $this->validacaoSalvaCabecalhoEntrada($rowLancamento);
            
            $this->dataSourceName = 'notafiscal';
            
            $rowLancamento["DTCOMPFATUR"] = date("d/m/Y H:i:s");
            $rowLancamento["CDOPERGERA"]  = $this->environment->getCdOperador();
            $request = new Request\Row($rowLancamento, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
        
            // Salva o cabeçalho
            parent::save($request, $response);
            
            
            
            $dataSets = $response->getDataSets();
            
            if(isset($dataSets[0])) {
            
                $rows = $dataSets[0]->getRows();
                if($rows[0]) {
                    $notaFiscal = $rows[0];
                    
                    $filial     = $notaFiscal["CDFILIAL"];
                    $lancamento = $notaFiscal["NRLANCTONF"];
                        
                    if(!$verificou && $this->nfEntradaService->verificaNotaSendoUtilizada($filial, $lancamento)) {
                        throw new \Exception('Operação bloqueada. A nota fiscal está sendo utilizada por outro operador!');
                    }
                    
                    $rowLancamento["CDFILIAL"]      = $filial;
                    $rowLancamento["NRLANCTONF"]    = $lancamento;
                    $importouChave                  = false;
                    
                    
                    
                    
                    // Realiza as importações possiveis para a nota caso seja uma nova nota
                    if($rowLancamento["__is_new"] === true) {
                        if($rowLancamento["NRACESSONFE"] && $temAf) {
                            $this->nfEntradaService->setNotafiscalEntity($filial, $lancamento);
                            
                            $this->nfEntradaService->realizaImportacaoAfChaveAcesso($filial, $lancamento, $nracessonfe, $rowLancamento["AF"], $rowLancamento);
                            
                            $this->nfEntradaService->atualizaIntgNotaFiscal('U', 'S', 'S', $rowLancamento["CDFILIRECE"], $rowLancamento["CDFILIAL"], $rowLancamento["NRLANCTONF"], $rowLancamento["NRACESSONFE"]);
                            $this->nfEntradaService->atualizaNotaDestinada($filial, $lancamento, $rowLancamento["NRACESSONFE"]);
                            
                            $sequencial = $this->nfEntradaService->getSequencialHistoricoXML($rowLancamento["NRACESSONFE"])[0]["NRSEQHISTOR"];
                            if($sequencial === null) $sequencial = 1;
                            else $sequencial += 1;
                            $sequencial = str_pad($sequencial, 20, "0", STR_PAD_LEFT);
                            $dsOperacao = 'Inclusão de Nota a partir de arquivo XML';
                            $this->nfEntradaService->insereHistoricoXML($rowLancamento["NRACESSONFE"], $sequencial, date('d/m/Y H:i:s'), '2', 'S', '', 
                                            $dsOperacao, $rowLancamento["CDFILIAL"], $rowLancamento["NRLANCTONF"]);
                            
                            $importouChave = true;
                            
                        }
                        else if($rowLancamento["NRACESSONFE"]) {
                            $this->nfEntradaService->realizaImportacaoChaveAcesso($filial, $lancamento, $nracessonfe, $rowLancamento);   
                            // $this->nfEntradaService->atualizaIntgNotaFiscal($filial, $lancamento, $nracessonfe);
                            // $this->nfEntradaService->atualizaNotaDestinada($filial, $lancamento, $nracessonfe);
                            $this->nfEntradaService->atualizaIntgNotaFiscal('U', 'S', 'S', $rowLancamento["CDFILIRECE"], $rowLancamento["CDFILIAL"], $rowLancamento["NRLANCTONF"], $rowLancamento["NRACESSONFE"]);
                            $this->nfEntradaService->atualizaNotaDestinada($filial, $lancamento, $rowLancamento["NRACESSONFE"]);
                            $sequencial = $this->nfEntradaService->getSequencialHistoricoXML($rowLancamento["NRACESSONFE"])[0]["NRSEQHISTOR"];
      
                            if($sequencial === null) $sequencial = 1;
                            else $sequencial += 1;
                            $sequencial = str_pad($sequencial, 20, "0", STR_PAD_LEFT);
                            $dsOperacao = 'Inclusão de Nota a partir de arquivo XML';
                            $this->nfEntradaService->insereHistoricoXML($rowLancamento["NRACESSONFE"], $sequencial, date('d/m/Y H:i:s'), '2', 'S', '', 
                                            $dsOperacao, $rowLancamento["CDFILIAL"], $rowLancamento["NRLANCTONF"]);
                            $importouChave = true;
                        }
                        else if($temAf) {
                            $this->nfEntradaService->setNotafiscalEntity($rowLancamento['CDFILIAL'], $rowLancamento['NRLANCTONF']);
                            $this->nfEntradaService->realizaImportacaoAf($rowLancamento, $rowLancamento["AF"]);
                        }
                        
                        if(isset($rowLancamento["PKNOTAFISCORIG"])) {
                            if(is_array($rowLancamento["PKNOTAFISCORIG"])) {
                                $lancamentosOrig = array();
                                foreach($rowLancamento["PKNOTAFISCORIG"] as $notaOrigem ) {
                                    
                                    $notaOrig = explode("-", $notaOrigem);
                                    $filiOrig = $notaOrig[0];
                                    $lancOrig = $notaOrig[1];
                                    
                                    $this->nfEntradaService->insereItemNotaFisc($filial, $lancamento, $filiOrig, $lancOrig);
                                    if($rowLancamento["CDTIPOOPER"] != "03"){
                                        $itens = $this->nfEntradaService->getItensNotaOrigem($lancOrig, $filiOrig, $lancamento, $filial);
                                        
                                        $impostos = $this->nfEntradaService->salvaMultiItensLancamentoEntrada($itens, array(), $rowLancamento);
                                
                                    }
                                }
                                
                            } else {
                                $notaOrig = explode("-", $rowLancamento["PKNOTAFISCORIG"]);
                                $filiOrig = $notaOrig[0];
                                $lancOrig = $notaOrig[1];
                            
            
                                $this->nfEntradaService->insereItemNotaFisc($filial, $lancamento, $filiOrig, $lancOrig);
                                
                                if($rowLancamento["CDTIPOOPER"] != "03"){
                                    $itens = $this->nfEntradaService->getItensNotaOrigem($lancOrig, $filiOrig,$lancamento, $filial);
                                    $impostos = $this->nfEntradaService->salvaMultiItensLancamentoEntrada($itens, array(), $rowLancamento);
                            
                                }
                            }

                        }
                        
                        $vencimentos = $this->nfEntradaService->getVencimentos($filial, $lancamento);
                    }
                    else {
                        if(isset($rowLancamento["ATUALIZAINDICE"])) {
                            if($rowLancamento["ATUALIZAINDICE"]) {
                                $this->nfEntradaService->atualizaVrItemProduto($rowLancamento);
                            }
                        }
                    }
                    $commit = true;
                }
                
                try {
                    $dadosVenc = $this->nfEntradaService->getDadosVenc($rowLancamento["CDTIPOOPER"], $rowLancamento["IDENTRSAIDOP"], $rowLancamento["CDFORNECED"]);
                    $dadosVenc["IDUTILVENC"] = isset($dadosVenc["IDUTILVENC"]) ? $dadosVenc["IDUTILVENC"] : null;
                    $dadosVenc["IDGERAVENC"] = isset($dadosVenc["IDGERAVENC"]) ? $dadosVenc["IDGERAVENC"] : null;
                    
                    if($dadosVenc["IDUTILVENC"] != 'N' && empty($vencimentos) && $dadosVenc["IDGERAVENC"]) {
                        
                        if($rowLancamento["IDORIGDESTNF"] == 'F' || !$rowLancamento["IDORIGDESTNF"]) {
                            if($rowLancamento["VRNOTAFISC"] != 0) {
                                $this->nfEntradaService->gravaVencimento($rowLancamento, $dadosVenc);
                            }
                        }
                    }
                }
                catch(\Exception $e) {
                    throw $e;
                }
                
            }
            
            if(!$isApp) {
                $connection->commit();
                
                //Salva dados de compra
                $this->nfEntradaService->salvaInfoCompra($rowLancamento);
                $commit = true;
                //$connection->commit();
            }
            
        }
        catch(\Exception $e) {
            $connection->rollback();
            $response->setError(new Error($e->getMessage(), 400));
            $response->setStatus('400');
        }
        
        if($isApp && $commit) {
            try {
                $connection->commit();
                if($rowLancamento["__is_new"] === false) {
                    $vencimentos = $this->nfEntradaService->getVencimentos($filial, $lancamento);
                }
                $vencimentos = new \Zeedhi\Framework\DataSource\DataSet('contaspagar', $vencimentos);
                $response->addDataset($vencimentos);
            }
            catch(\Exception $e) {
                $connection->rollback();
                $response->setError(new Error($e->getMessage(), 400));
                $response->setStatus('400');
            }
        }
        
    }
    
    public function salvarNotaEntradaDevolucao(Request\Row $request, Response $response) {
        $rowLancamento = $request->getRow();
        
        $connection = $this->entityManager->getConnection();
        
        $isApp  = isset($rowLancamento["APP"]) ? $rowLancamento["APP"] : false;
        $commit = false;

        $verificou = false;

        if(isset($rowLancamento["NRLANCTONF"])) {
            if($this->nfEntradaService->verificaNotaSendoUtilizada($rowLancamento["CDFILIAL"], $rowLancamento["NRLANCTONF"])) {
                throw new \Exception('Operação bloqueada. A nota fiscal está sendo utilizada por outro operador!');
            }
            else{
                $verificou = true;
            }
        }
        
        try {
            
            $connection->beginTransaction();
             
            $rowLancamento = $request->getRow();
            
            $nracessonfe = $rowLancamento["NRACESSONFE"];
            $sequencial = $this->nfEntradaService->getSequencialHistoricoXML($rowLancamento["NRACESSONFE"])[0]["NRSEQHISTOR"];
            if(isset($rowLancamento["AF"])) {
                if(!empty($rowLancamento["AF"])) {
                    $temAf = true;
                }
                else {
                    $temAf = false;
                }
            }
            else {
                $temAf = false;
            }
            
            // Faz as validações necessarias para salvar o cabeçalho
            $this->validacaoSalvaCabecalhoEntrada($rowLancamento);
            
            $this->dataSourceName = 'notafiscal';
            
            $rowLancamento["DTCOMPFATUR"] = date("d/m/Y H:i:s");
            $rowLancamento["CDOPERGERA"]  = $this->environment->getCdOperador();
            $request = new Request\Row($rowLancamento, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
        
            // Salva o cabeçalho
            parent::save($request, $response);
            
            $dataSets = $response->getDataSets();
            
            if(isset($dataSets[0])) {
            
                $rows = $dataSets[0]->getRows();
                if($rows[0]) {
                    $notaFiscal = $rows[0];
                    
                    $filial     = $notaFiscal["CDFILIAL"];
                    $lancamento = $notaFiscal["NRLANCTONF"];
                        
                    if(!$verificou && $this->nfEntradaService->verificaNotaSendoUtilizada($filial, $lancamento)) {
                        throw new \Exception('Operação bloqueada. A nota fiscal está sendo utilizada por outro operador!');
                    }
                    
                    $rowLancamento["CDFILIAL"]      = $filial;
                    $rowLancamento["NRLANCTONF"]    = $lancamento;
                    $importouChave                  = false;
                    
                    $produtos = $rowLancamento["PRODUTOS"];
                    
                    if($rowLancamento["__is_new"] === true){
                        $this->nfEntradaService->salvaItensDevolucao($produtos, $rowLancamento, $filial, $lancamento);
                        $commit = true;
                        
                        if(isset($rowLancamento["PKNOTAFISCORIG"])) {
                            if(is_array($rowLancamento["PKNOTAFISCORIG"])) {
                                $lancamentosOrig = array();
                                foreach($rowLancamento["PKNOTAFISCORIG"] as $notaOrigem ) {
                                    
                                    $notaOrig = explode("-", $notaOrigem);
                                    $filiOrig = $notaOrig[0];
                                    $lancOrig = $notaOrig[1];
                                    
                                    $this->nfEntradaService->insereItemNotaFisc($filial, $lancamento, $filiOrig, $lancOrig);
                                }
                            }else {
                                $notaOrig = explode("-", $rowLancamento["PKNOTAFISCORIG"]);
                                $filiOrig = $notaOrig[0];
                                $lancOrig = $notaOrig[1];
                            
            
                                $this->nfEntradaService->insereItemNotaFisc($filial, $lancamento, $filiOrig, $lancOrig);
                                }
                            }
                    }
                }
                
            }
            
            if(!$isApp) {
                $connection->commit();
                
                //Salva dados de compra
                $this->nfEntradaService->salvaInfoCompra($rowLancamento);
                $commit = true;
                //$connection->commit();
            }
            
        }
        catch(\Exception $e) {
            $connection->rollback();
            $response->setError(new Error($e->getMessage(), 400));
            $response->setStatus('400');
        }
        
        if($isApp && $commit) {
            try {
                $connection->commit();
                if($rowLancamento["__is_new"] === false) {
                    $vencimentos = $this->nfEntradaService->getVencimentos($filial, $lancamento);
                }
                $vencimentos = new \Zeedhi\Framework\DataSource\DataSet('contaspagar', $vencimentos);
                $response->addDataset($vencimentos);
            }
            catch(\Exception $e) {
                $connection->rollback();
                $response->setError(new Error($e->getMessage(), 400));
                $response->setStatus('400');
            }
        }
        
    }

    protected function validacaoSalvaCabecalhoEntrada($rowLancamento) {
        
        $filial = $rowLancamento['CDFILIRECE'];
        $dtLancamento = $rowLancamento['DTENTRSAID'];
        $operacao = 'E';
        $sistema = 'DFE';
        $dtEmissao = $rowLancamento['DTEMISSAO'];
        $nrnotafisc = $rowLancamento['NRNOTAFISC'];
        $nrinsjurnf = $rowLancamento['NRINSJURNF'];
        $serie = \Zeedhi\Framework\Util\Functions::arrayKeyExists('CDSERIEENTR', $rowLancamento) ? $rowLancamento['CDSERIEENTR'] : null;
        $nracessonfe = $rowLancamento["NRACESSONFE"];
        
        if($rowLancamento["NRACESSONFE"] && $rowLancamento["IDNFGERADAVIAXML"] == 'N') {
            $this->nfEntradaService->validaCamposChaveAcesso($rowLancamento);
        }
        
        $this->validacoesEstoqueService->validaDataLancamento($filial, $dtLancamento, $operacao, $sistema, $dtEmissao);
        
        if($dtLancamento && $dtEmissao) {
            if(strtotime(str_replace("/", "-", $dtEmissao)) > strtotime(str_replace("/", "-", $dtLancamento))) {
                throw new \Exception("Data de entrada deve ser superiror a data de emissão.");
            }
        }

        $this->nfEntradaService->validaNotaDeOrigem($rowLancamento);
        
        if(isset($rowLancamento["NRLANCTONF"])) {
            $this->nfEntradaService->limpaCapa($filial, $rowLancamento["NRLANCTONF"]);
        }
        //não validar para 10 e 9
        if ($rowLancamento["CDTIPOOPER"] != '10' && $rowLancamento["CDTIPOOPER"] != '09'){
            if(isset($rowLancamento["CDMODDOCFISC"])){
                $modeloDocumento = $this->nfEntradaService->validaModeloDocumento($rowLancamento["IDENTRSAIDOP"], $rowLancamento["CDTIPOOPER"], $rowLancamento["CDMODDOCFISC"]);
                if(empty($modeloDocumento)) {
                    throw new \Exception("Modelo de documento invalido para o tipo de operação.");
                }  
            }else{
                throw new \Exception("Modelo de documento vazio.");
            }
            
        }
        
    }
    
     public function geraSenhaLancRetroativo(Request\Row $request, Response $response) {
         $row = $request->getRow();
         
         $cdfilial = intval($row["CDFILIAL"]);
         $nrlancto = intval($row["NRLANCTO"]);
         $data     = intval(date('dmY'));
         $sum      = sqrt($cdfilial + $nrlancto + $data);
         $sum     -= intval($sum);
         $senha = substr(strval($sum), 2, 6);
         $data = array( "SENHAGERADA" => $senha );
         
         $response->addDataset(new DataSource\DataSet('senha', $data));
     }


    public function realizaImportacaoChaveAcesso(Request\Row $request, Response $response) {
        try {
            $rowLancamento = $request->getRow();

            $filial = $rowLancamento['CDFILIAL'];
            $lancamento = $rowLancamento['NRLANCTONF'];
            $nracessonfe = $rowLancamento['NRACESSONFE'];

            $this->nfEntradaService->realizaImportacaoChaveAcesso($filial, $lancamento, $nracessonfe);

        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function realizaImportacaoAfChaveAcesso(Request\Row $request, Response $response) {
        try {
            $rowLancamento = $request->getRow();

            $filial = $rowLancamento['CDFILIAL'];
            $lancamento = $rowLancamento['NRLANCTONF'];
            $nracessonfe = $rowLancamento['NRACESSONFE'];

            if (\Zeedhi\Framework\Util\Functions::arrayKeyExists('AF', $rowLancamento)) {
                $afSelecionadas = $rowLancamento['AF'];
            }

            $this->nfEntradaService->setNotafiscalEntity($filial, $lancamento);
            $this->nfEntradaService->realizaImportacaoAfChaveAcesso($filial, $lancamento, $nracessonfe, $afSelecionadas);

        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function salvaRelacionamentoItensXml(Request\Row $request, Response $response) {
        try {
            $rowLancamento = $request->getRow();

            $cdprodutointg = $rowLancamento['CDPRODUTOINTG'];
            $cdforneced    = $rowLancamento['CDFORNECED'];
            $cdprodfornxml = $rowLancamento['CDPRODFORNXML'];
            $nmprodutointg = $rowLancamento['NMPRODUTOINTG'];
            $cdfilial      = $rowLancamento["CDFILIAL"];
            $nracessonfe   = $rowLancamento['NRACESSONFE'];
            $CDPRODUTOXML_BACKUP_ROW =  $rowLancamento["CDPRODUTOXML_BACKUP_ROW"];
           
           $this->nfEntradaService->vinculaNcm($cdprodutointg,$cdprodfornxml,$nracessonfe);

            $this->nfEntradaService->salvaRelacionamentoItensXml(
                $cdprodutointg, $cdforneced, $cdprodfornxml, $nmprodutointg, $cdfilial, $CDPRODUTOXML_BACKUP_ROW
            );

        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function removerRelacionamentoItemXml(Request\Row $request, Response $response) {
        try {
            $rowLancamento = $request->getRow();
            $cdforneced = $rowLancamento['CDFORNECED'];
            $cdprodutoxml = $rowLancamento['CDPRODUTOXML'];
            $cdfilial      = $rowLancamento['CDFILIAL'];
            $cdproduto =  $rowLancamento["CDPRODUTO"];


            $this->nfEntradaService->removerRelacionamentoItemXml(
                $cdforneced, $cdprodutoxml, $cdfilial, $cdproduto
            );

        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function salvaMultiItensLancamentoEntrada(Request\DataSet $request, Response $response) {
        try {
            $rows = $request->getDataSet()->getRows();
            $novosItens = $rows[0];
            $itensAlterados = $rows[1];
            $cabecalho = $rows[2];

            $impostos = $this->nfEntradaService->salvaMultiItensLancamentoEntrada($novosItens, $itensAlterados, $cabecalho);

            $response->addDataset(new DataSource\DataSet('impostosTemp', $impostos));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    
    public function getDefaultFilterLancamentoEntrada(Request\Row $request, Response $response) {
        $row = $request->getRow();
        
        $defaultFilter = $this->nfEntradaService->getDefaultFilter($row);
        
        $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('defaultFilter', $defaultFilter));
    }
    
    public function getDefaultNovaNota(Request\Row $request, Response $response) {
        $row = $request->getRow();
        
        $cdFilial = isset($row["CDFILIAL"]) ? $row["CDFILIAL"] : ''; 
        
        $defaultFilter = $this->nfEntradaService->getDefaultNovaNota($row["CDFILIAL"]);
        
        $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('defaultNovaNota', $defaultFilter));
    }
    
    public function getIDATUESTTPOP(Request\Row $request, Response $response) {
        $row = $request->getRow();
        
        //$cdFilial = isset($row["CDFILIAL"]) ? $row["CDFILIAL"] : ''; 
        
        $atualizaEstoque = $this->nfEntradaService->getIDATUESTTPOP($row);
        
        //$response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('atualizaEstoque', $atualizaEstoque));
        $response->addDataset(new DataSource\DataSet('atualizaEstoque', $atualizaEstoque));
    }
    
    public function getParamsRelacItens(Request\Row $request, Response $response) {
        $row = $request->getRow();
        
        $cdfilial = $row["CDFILIAL"];
        $cdforneced = $row["CDFORNECED"];
        
        $relacItensParams = $this->nfEntradaService->getParamsRelacItens($cdfilial, $cdforneced);
        
        $response->addDataset(new DataSource\DataSet('relacItensParams', $relacItensParams));
    }
    
    public function atualizaItemAf(Request\Row $request, Response $response) {
        $item = $request->getRow();
        $this->nfEntradaService->atualizaItemAf($item);
    }
    
    public function excluirItemAf(Request\Row $request, Response $response) {
        $item = $request->getRow();
        
        $this->nfEntradaService->excluirItemAf($item);
    }
    
    public function generateReportConsistencia(Request\Row $request, Response $response) {
        var_dump($this->reportService);
    }
    
    public function atualizaVrItemProduto(Request\Row $request, Response $response) {
         $rows =  $request->getRow();
         try{
            $this->nfEntradaService->atualizaVrItemProduto($rows);
         } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    
    public function consultaNotaUtilizaAfIndice(Request\Row $request, Response $response) {
         $nota =  $request->getRow();
         try{
            $utilizaIndice = $this->nfEntradaService->consultaNotaUtilizaAfIndice($nota);
            $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('af_indice', ['utilizaIndice' => $utilizaIndice]));
         } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getAnexoNfe(Request\Filter $request, Response $response) {
        $this->dataSourceName = 'anexonfe';
        parent::find($request, $response);
        
        $dataSets = $response->getDataSets();
        $rows = isset($dataSets[0])? $dataSets[0]->getRows() : $dataSets;
        foreach ($rows as &$row) {
            $link = $row["DSLINKANEXO"];
            $dslinkanexo = explode(">", $row["DSLINKANEXO"]);
            if(isset($dslinkanexo[0])){
                $row["DSLINKANEXO"] = str_replace("<", "", $dslinkanexo[0]);
                $row["link"] = $this->fileServerBaseUrl.str_replace(">", "", $dslinkanexo[1]);
            }
        }
        
        $dataSet = new DataSet('anexonfe', $rows);
        $response->setDataSets(array($dataSet));
    } 
    
    public function deletarAnexo(Request\Row $request, Response $response) {
        try {
            $this->dataSourceName = 'anexonfe';
            $row = $request->getRow();
            $path = explode("files/",$row['link'])[1];

            $this->nfEntradaService->deletarAnexoFileServer($path);
            parent::delete($request, $response);
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
     public function salvarAnexo(Request\Row $request, Response $response){
        try {
            $rows =  $request->getRow();
            $this->nfEntradaService->salvaAnexo($rows);
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
        
    }

    public function baixaAnexo(Request\Row $request, Response $response){
        try {
            $rows =  $request->getRow();
            $row[] = $this->nfEntradaService->baixaAnexo($rows);
            $response->addDataset(new DataSource\DataSet($row[0]));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
     public function salvarNotaOperacaoOnze(Request\DataSet $request, Response $response) {
        $rows = $request->getDataSet()->getRows();
        $this->nfEntradaService->insereNotasOpOnze($rows, $this->lancamentoNotafiscalService);
    }
    
    public function validaParamProduto(Request\Row $request, Response $response){
        try {
            $row =  $request->getRow();
            
            $paramProduto = $this->nfEntradaService->validaParamProduto($row);
            $response->addDataset(new DataSource\DataSet('validaParamProduto', array($paramProduto)));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function novoImpostoItemTemp(Request\DataSet $request, Response $response) {
         try {
            $rows = $request->getDataSet()->getRows();
            $this->nfEntradaService->novoImpostoItemTemp($rows);
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function verificaControladoria(Request\Row $request, Response $response) {
        $utilizaControladoria[0] = $this->nfEntradaService->verificaControladoria();
        $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('utilizaControladoria', $utilizaControladoria));
    }
    
    public function validaNotaCadastrada(Request\Row $request, Response $response){
        try {
            $row =  $request->getRow();
            
            
            //comentario para gerar conteudo - resolver conflito
            if($row['__is_new']) {
                $this->nfEntradaService->validaNotaCadastrada($row['CDFILIRECE'], $row['CDSERIEENTR'], $row['NRINSJURNF'], $row['NRNOTAFISC']);
            }else{
                $this->nfEntradaService->validaEditNotaCadastrada($row['CDFILIRECE'], $row['CDSERIEENTR'], $row['NRINSJURNF'], $row['NRNOTAFISC'], $row["NRLANCTONF"]);
            }
        
        
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    
    public function lancamentoNfSaida(Request $request, Response $response){
        $this->novaNfSaida = (object)[];
        $commit = false;
        
        try{
            $row = $request->getRow();
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            //variáveis utilizadas
            $serieSaida; $itensNfEntr; $params;
            
            $nf = $request->getRow()['nfEntrada'];
            
            $nfSaida = $request->getRow()['nfSaida'];
            $nfSaida["msg"] = false;
            
            // % venda
            $nfSaida["porcentagemVenda"] = ($nfSaida["porcentagemVenda"]/100);
            
            //impostos da entrada
            $impostos = $nf["impostos"];
            
            $valorImpostoAcresce = $this->somatorioImpostosAcrescem($impostos);
            
            //verfica se existe a serie na saida == entrada
            $serieSaida = $this->nfEntradaService->verificaSerie($nf);
            if(!$serieSaida){
                throw new \Exception ('Não existe série para gerar a Nf de saída que corresponde a Nf de entrada. Conferir parametrização das series de entrada e saída. Série da nota de entrada: '. $nf["CDSERIEENTR"]);
            }else{
                $nfSaida["CDSERIE"] = $serieSaida["CDSERIE"];
                $nfSaida["DSSERIE"] = $serieSaida["DSSERIE"];
            }
    
            //verifica se existem produtos na nota de entrada
            if(!$nf["produtos"]){
                throw new \Exception ('Não existem produtos para gerar a Nf de saída que corresponde a Nf de entrada. Conferir lançamento dos produtos na tela de lançamento de entrada. Nf de entrada: ' . $nf["NRLANCTONF"]);
            }else{
                $nfSaida["produtos"] = $this->nfEntradaService->buscaProdutosNfEntrada($nf["CDFILIAL"], $nf["NRLANCTONF"]);
            }
            
            //busca os encargos da nf de entrada que serão utilizados na saída;
            $nf["encargos"] = $this->nfEntradaService->encargosNfEntrada($nf);
            $existeEncargo = $this->verficaExistenciaEncargo($nf["encargos"]);
            
            //parametros fiscais de entrada -> campo IDUTVRCONTPRORIG;
            $paramFilial = $this->nfEntradaService->getParamFilial($nf);
            $paramFilial = $paramFilial[0];
            if($paramFilial["IDUTVRCONTPRORIG"]  == 'N'){
                
                //caso tenha sido informado algum valor no campo -> porcentagemVenda, considerado para calculo do novo encargo;
                if($nfSaida["porcentagemVenda"] != 0){
                    //VRUNITITEM -> fórmula; VRDESCITEM -> = Entrada; Encargos -> * % de venda;
                    $nfSaida["produtos"] = $this->calculaVrUnitario_formula($nfSaida["produtos"], $valorImpostoAcresce, false, $nfSaida["porcentagemVenda"]);
                    $nfSaida["produtos"] = $this->getDescontosEntrada($nfSaida["produtos"], $nf["produtos"]);
                    $existeEncargo ? $nfSaida["encargos"] = $this->encargoXpercentual_calcula($nf["encargos"], $nfSaida["porcentagemVenda"]) : $nfSaida["encargos"] = [];
                }else{
                    // mantem o encargo e o valor unitario dos produtos
                    $existeEncargo ? $nfSaida["encargos"] = $nf["encargos"] : $nfSaida["encargos"] = [];
                }
                
                    
            }else{
                $nfSaida["produtos"] = $this->calculaVrUnitario_formula($nfSaida["produtos"], 0, true, 0);
            }
            
            $nfSaida = $this->ajustaNfSaida($nfSaida, $nf);
            
            //geraNotaFiscalSaida: -> retorna: erros, nota de saida e os produtos;
            //fechaNotaFiscalSaida: fecha a nota de saida e o rateio;
            $retorno = $this->nfEntradaService->geraNotaFiscalSaida($nfSaida, $nf, $this->lancamentoNotafiscalService);
            if(count($retorno["erros"]) > 0) return $response->addDataset(new DataSource\DataSet('lancNfSaida', array("erros" => $retorno["erros"])));
            
            $this->novaNfSaida = $this->insereInfosTipoOp($retorno["nfSaida"], $nfSaida);
            $produtosSaida = $retorno["itens"];
            
            $this->nfEntradaService->insereEncargosSaida($this->novaNfSaida["CDFILIAL"], $this->novaNfSaida["NRLANCTONF"], $nfSaida["encargos"], 'N', 'S', $this->novaNfSaida["NRORG"]);
            
            $this->nfEntradaService->fechaNotaFiscalSaida($this->novaNfSaida, $this->lancamentoNotafiscalService, $nfSaida);
            
            $impostObg = $this->nfEntradaService->verifImpObrig($this->novaNfSaida["CDFILIEMIT"], $this->novaNfSaida["CDFILIAL"], $this->novaNfSaida["NRLANCTONF"], $this->lancamentoNotafiscalService);
            
            if(!empty($impostObg)){
                $e = $this->erroImpoObg($impostObg);
                $e = $e[0]["ERROIMPOPROD"].". ".$e[0]["NMPRODUTO"];
                throw new \Exception ($e);
            }
            
            $connection->commit();
            
            $this->salva_DadosAd_ItemNotaFisc($this->novaNfSaida["CDFILIAL"], $this->novaNfSaida["NRLANCTONF"], $nf["CDFILIAL"], $nf["NRLANCTONF"]);
            
            //params -> controla estoque e realiza atualização on-line da movimentação de estoque, atualização de estoque
            $params["IDESTONLINE"] = $paramFilial["IDESTONLINE"];
            $params["IDCTRLESTQ"] = $paramFilial["IDCTRLESTQ"];
            $params["IDATUESTTPOP"] = $nfSaida["IDATUESTTPOP"];
            
            if ($params["IDCTRLESTQ"] == 'S' && $nfSaida["IDATUESTTPOP"] == 'S') {
               if ($params["IDESTONLINE"] != 'N') {
                   $this->nfEntradaService->atualizaEstoque($this->novaNfSaida["CDFILIAL"], $this->novaNfSaida["NRLANCTONF"], $this->lancamentoNotafiscalService);
               }
            }
            
            //{Atualizar estoque da nota fiscal de entrada da filial fornecedor}
            $commit = true;
            
        }catch (\Exception $e) {
            $this->erros = array($e->getMessage());
            $connection->rollback();
        }
        
        if($commit) $this->atualizaEstoqueEntrada($this->novaNfSaida["CDFILIAL"], $this->novaNfSaida["NRLANCTONF"]);
        $response->addDataset(new DataSource\DataSet('lancNfSaida', array("erros" => $this->erros, "nfSaida" => $this->novaNfSaida)));
        
    }
    
    private function somatorioImpostosAcrescem($impostos){
        //aprox de 2 casas
        $valorImpostoAcresce = 0;
        
        foreach($impostos as $imp){
            if($imp["IDTRATIMPO"] == "A") $valorImpostoAcresce .= $imp["VRIMPOSTONF"];
        }
        
        $valorImpostoAcresce = round($valorImpostoAcresce, 2);
        
        return $valorImpostoAcresce; 
    }
    
    private function calculaVrUnitario_formula($produtos, $valorImpostoAcresce, $IDUTVRCONTPRORIG = false, $porcentagemVenda){
        unset($this->act);
        
        if(!$IDUTVRCONTPRORIG){
            foreach($produtos as $prod){
                if($prod["QTITEMPROD"] == 0) {
                    return $prod["VRUNITITEM"] = 0; 
                }
                
                if($valorImpostoAcresce == 0){ 
                    $prod["VRUNITITEM"] = round($prod["VRUNITITEM"] * $porcentagemVenda, 2);
                }else{
                    $prod["VRUNITITEM"] = round(($prod["VRUNITITEM"] * $porcentagemVenda) + (($valorImpostoAcresce * $porcentagemVenda) / $prod["QTITEMPROD"]), 2);
                }
                
                $this->act[] = $prod;
            }
            
            return $this->act;
            
        }else{
            
            //VRUNITITEM será mantido 
            //VRDESCITEM = 0,
            foreach($produtos as $prod){
                $prod["VRDESCITEM"] = 0;
                $this->act[] = $prod;
            }
            return $this->act;
        }
    }
    
    private function encargoXpercentual_calcula($encargos, $percentual){
        unset($this->act);
        foreach($encargos as $encargo){
            if($encargo["VRITEMENCA"] > 0){
                $encargo["VRITEMENCA"] = round($encargo["VRITEMENCA"] * $percentual, 2);
            }
            
            $this->act[] = $encargo;
        }
        
        return $this->act;
    }
    
    private function getDescontosEntrada($saida, $entrada){
        unset($this->act);
        foreach($entrada as $produtoE){
            foreach($saida as $produtoS){
                $produtoS["VRDESCITEM"] = $produtoE["VRDESCITEM"];
                $this->act[] = $produtoS;
            }
        }
        
        return $this->act;
    }
    
    private function montaCorpoMsgDadosAdicionais($filial){
       unset($this->act);
       // nome
       $nome = $this->nfEntradaService->getNmfilial($filial);
       // cnpj
       $cnpj = $this->nfEntradaService->getCnpj($filial);
       // endereço
       $ende = $this->nfEntradaService->getEndereco($filial);
       // municipio
       $mncp = $this->nfEntradaService->getMunicipio($filial)->getNmmunicipio();
       // estado
       $sg = $this->nfEntradaService->getSg($filial);
       // ie
       $ie = $this->nfEntradaService->getIe($filial)->getCdinscesta();
       
       $msg = "Mercadoria será entregue pela empresa {$nome}, estabelecida no endereço {$ende} - {$mncp}/{$sg}, CNPJ: {$cnpj} e IE: {$ie}";
       
       return $msg;
    }
    
    private function verficaExistenciaEncargo($encargos){
        $count = 0;
        
        foreach($encargos as $encargo){
            if($encargo["VRITEMENCA"] > 0){
                $count++;
                $encargo["VRITEMENCA"] = round($encargo["VRITEMENCA"], 2);
            }
        }
        
        if($count == 0){
            return false;
        }else{
            return true;
        }
    }
    
    //chamar sempre -> fazer rota
    public function salva_DadosAd_ItemNotaFisc($CDFILIAL, $nrlantonf, $cdfilialit, $nrlantonfit){
        $this->gravaDadosAdicionais($nrlantonf, $CDFILIAL);
        $this->gravaItemNotaFisc($CDFILIAL, $nrlantonf, $cdfilialit, $nrlantonfit);
    }
    
    private function gravaDadosAdicionais($nrlantonf, $filial){
        $msg = $this->montaCorpoMsgDadosAdicionais($filial);
        $this->nfEntradaService->salvaMsgDadosAd($nrlantonf, $filial, $msg);
    }
    
    private function gravaItemNotaFisc($CDFILIAL, $nrlantonf, $cdfilialit, $nrlantonfit){
        //insert itemnotafisc
        $this->nfEntradaService->insereItemNotaFisc($CDFILIAL, $nrlantonf, $cdfilialit, $nrlantonfit);
        
    }
    
    private function ajustaNfSaida($nfSaida, $nfEntrada){
        $nfSaida["CDFILIAL"] =  $nfEntrada["CDFILIAL"];  
        $nfSaida["CDFILIEMIT"] =  $nfEntrada["CDFILIAL"];  
        $nfSaida["CDEMPRESA"] =  $nfEntrada["CDEMPRESA"];
        $nfSaida["CDTIPOOPER"] =  $nfSaida["CDTIPOOPER"];
        $nfSaida["CDSERIE"] =  $nfEntrada["CDSERIEENTR"];
        $nfSaida["DTENTRSAID"] = date('d/m/Y');
        $nfSaida["DSMODDOCFISC"] = 'Nota Fiscal Eletrônica';
        $nfSaida["IDSISTEMA"] = 'DFS';
        $nfSaida["CDMODDOCFISC"] = '55';
        $nfSaida["IDENTRSAIDOP"] = 'S';
        $nfSaida["IDBLOQUEADO"] = 'N';
        $nfSaida["NRLANCTONF"] = '';
        $nfSaida["IDSTATUS"] = 'D';
        $nfSaida["CDINSCESTA"] = $this->nfEntradaService->getIe($nfSaida["CDFILIAL"])->getCdinscesta();
        $vr = $this->calculaVr_saida($nfSaida["produtos"]);
        $nfSaida["VRNOTAFISC"] = $vr;
        $nfSaida["VRPRODUTOS"] = $vr;
        
        switch ($nfSaida["IDORIGDESTOP"]) {
            case 'C':
                $nfSaida["CDCLIENTE"] =  $nfSaida["CDCLIENTE"];
                $nfSaida["IDTPIJURNF"] = $nfSaida["IDTPIJURNF"];
                break;
                
            case 'F':
                $nfSaida["CDCLIENTE"] =  $nfSaida["CDFORNECED"];
                $nfSaida["IDTPIJURNF"] = $nfSaida["IDTPIJURNF"];
                break;
                
            case 'L':
                $nfSaida["CDFILIRECE"] =  $nfSaida["CDFILIRECE"];
                $nfSaida["CDCLIENTE"] = $nfSaida["CDFILIRECE"];
                break;
        }
        
        if($nfSaida["CDTIPOOPER"] == '06'){
            $nfSaida["CDFILINOTAORIG"] = $nfEntrada["CDFILIAL"];
            $nfSaida["NRLANCNOTAORIG"] = $nfEntrada["NRLANCTONF"];
        }
        
        return $nfSaida;
    }
    
    private function calculaVr_saida($produtos){
        $VRNOTAFISC = 0;
        foreach($produtos as $produto){
            $VRNOTAFISC = $VRNOTAFISC + ($produto["QTITEMPROD"] * $produto["VRUNITITEM"]);
        }
        
        return $VRNOTAFISC;
    }
    
    private function insereInfosTipoOp($nf, $nfSaida){
        
        $nf["IDUTILVENCOBR"] = $nfSaida["IDUTILVENCOBR"];
        $nf["IDNFORIGOBRI"] = $nfSaida["IDNFORIGOBRI"];
        $nf["IDOPERDEVONF"] = $nfSaida["IDOPERDEVONF"];
        $nf["IDORIGDESTOP"] = $nfSaida["IDORIGDESTOP"];
        $nf["IDPERNUMENF"] = $nfSaida["IDPERNUMENF"];
        $nf["IDSERIEAUTO"] = $nfSaida["IDSERIEAUTO"];
        $nf["IDNFAJUSTE"] = $nfSaida["IDNFAJUSTE"];
        $nf["IDUTILNFSE"] = $nfSaida["IDUTILNFSE"];
        $nf["IDUTILVENC"] = $nfSaida["IDUTILVENC"];
     
        return $nf;   
    }
    
    private function atualizaEstoqueEntrada($filialit, $NRLANCTONFIT){
        $adqExisteNFRelac = $this->nfEntradaService->adqExisteNFRelac($filialit, $NRLANCTONFIT);
        
        if(!empty($adqExisteNFRelac)){
            $adqExisteLancEstq = $this->nfEntradaService->adqExisteLancEstq($adqExisteNFRelac["CDFILIAL"], $adqExisteNFRelac["NRLANCTONF"]);
            if(!empty($adqExisteLancEstq)){
                 $this->nfEntradaService->atualizaEstoque($adqExisteLancEstq["CDFILIMOVI"], $adqExisteLancEstq["NRLANCESTQ"], $this->lancamentoNotafiscalService);
            }
        }
    }
    
    public function erroImpoObg($impostObg){
            $produtosSemImposto = array();
            foreach($impostObg as $imposto){
               array_push($produtosSemImposto,$imposto);
            }
            return $produtosSemImposto;
    }
    
    public function atualizaAlmoxarifado(Request\Row $request, Response $response) {
        $rowRelacionamento = $request->getRow();
        
        
        $connection = $this->entityManager->getConnection();
        
        try {
            
            $connection->beginTransaction();
            $cdfilial = $rowRelacionamento[0]['CDFILIAL'];
            $nrlanctonf = $rowRelacionamento[0]['NRLANCTONF'];
            $almoxPadrao = $rowRelacionamento[0]['CDALMOXARIFE'];
            $localPadrao = $rowRelacionamento[0]['CDLOCALESTOQ'];
            $this->nfEntradaService->atualizaAlmoxarifado($cdfilial, $nrlanctonf,$rowRelacionamento, $almoxPadrao, $localPadrao);
            
            $connection->commit();
                
        }
        catch(\Exception $e) {
            $connection->rollback();
            $response->setError(new Error($e->getMessage(), 400));
            $response->setStatus('400');
        }
    }
    
    public function buscaParamIDPERIMPSPRAF(Request\Row $request, Response $response) {
        
        try {
            $IDPERIMPSPRAF = $this->nfEntradaService->buscaParamIDPERIMPSPRAF();
            if($IDPERIMPSPRAF == false){
                throw new \Exception('Param_Doc_Fisc_Entr_IDPERIMPSPRA');
            }else{
                $response->addDataset(new DataSource\DataSet('param_IDPERIMPSPRAF_paramintegra', array($IDPERIMPSPRAF)));
            }
            
        }
        catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 400));
            $response->setStatus('400');
        }
    }
    
    public function getDsnotaFiscOrigAllNfController(Request\Row $request, Response $response) {
        $rows = $request->getRow();
        try {
            
            $nfOrigem = $this->nfEntradaService->getDsnotaFiscOrigAllNfService($rows);
            $response->addDataset(new DataSource\DataSet('notaFiscalOrigem', array("notas_encontradas" => $nfOrigem)));
        }
        catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 400));
        }
    }
    
    public function deletaItemProduto(Request\Row $request, Response $response) {

        $item = $request->getRow();
        try{
            if($this->nfEntradaService->verificaAuditoriaItem($item["CDFILIAL"], $item["NRLANCTONF"], $item["NRSEQITPROD"])) {
                throw new \Exception('Existe análise/auditoria para o item selecionado');
            }
            $this->nfEntradaService->deletaItem($item);
            
            $response->addDataset(new DataSource\DataSet('delete', array("success" => "S")));
            
        }catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }
    
}
