<?php

namespace Teknisa\DF\Modules\Entrada\Services;

use Teknisa\DF\Modules\Entrada\Factories\Dfe47000LiberacaoParaFinanceiroFactory;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource;
use Teknisa\DF\Modules\Entrada\Services\ManifestacaoDestinatarioService;
use \Teknisa\Libs\Util\SMTP;

class Dfe47000LiberacaoParaFinanceiroService {
    
    protected $Dfe52000Factory;
    protected $Nrorg;
    protected $erros = array();
    protected $titulosSucesso = 0;
    protected $nfeTools;
    private $fields;

    public function __construct(Dfe47000LiberacaoParaFinanceiroFactory $dfe47000LiberacaoParaFinanceiroFactory, $environment, ManifestacaoDestinatarioService $manifestacaoDestinatarioService) {
        $this->dfe47000LiberacaoParaFinanceiroFactory = $dfe47000LiberacaoParaFinanceiroFactory;
        $this->nrorg        = $environment->getNrOrg();
        $this->filialLogada = $environment->getCdFilial();
        $this->environment  = $environment;
        $this->manifestacaoDestinatarioService = $manifestacaoDestinatarioService;
        
        $this->fields = new \stdClass();
        $this->fields->destinatario = '';
        $this->fields->data = '';
        $this->fields->numero = '';
        $this->fields->valor = 0;
        $this->fields->chave = '';
        $this->fields->data = '';
        $this->fields->correcao = '';
        $this->fields->conduso = '';
        $this->fields->chave   = '';
    }

    public function getFilter() {
        try {
            $dados = $this->dfe47000LiberacaoParaFinanceiroFactory->getDadosFilter($this->filialLogada);
            $row   = array( "CDEMPRESA" => $dados["CDEMPRESA"],
                            "NMEMPRESA" => $dados["NMEMPRESA"],
                            "DTENTRSAID" => $this->getDate($dados["DTPROCESSA"]));
        }
        catch(\Exception $e) {
             $row   = array("CDEMPRESA" => "",
                            "NMEMPRESA" => "",
                            "DTENTRSAID" => $this->getDate());
        }
        $row["TCDTIPOOPER"]     = "T";
        $row["TCDSERIEENTR"]    = "T";
        $row["TCDFILIORIG"]     = "T";
        $row["ERRO"]            = "T";
        $row["IDBLOQNFFIN"]     = "T";
        return $row;
    }
    
    public function getDate($dtProcessa = '') {
        if(!$dtProcessa) $dtProcessa = 'now';
        else $dtProcessa = str_replace("/", "-", substr($dtProcessa, 0, 10));
  
        $date   = new \DateTime($dtProcessa);
        $date->modify('last day of this month');
        $mes    = $date->format('m');
        $ano    = $date->format("Y"); 
        $date   = array("start" => "01"."/$mes/".$ano, "end" => $date->format('d/m/Y'));
        return $date;
    }
    
     /**
    *   Bloqueia ou libera o titulo de acordo como o parametro $mode
    *   
    *   @param array   $det                 - Dados Completos do produto (impostos, valores, etc)
    *   @param char    $mode                - B (Bloquear) - L (Liberar)
    */
    public function setAprovacao($notas, $mode) {
        $configs        = array();
        $paramManifesta = $this->dfe47000LiberacaoParaFinanceiroFactory->getParamManifesta(); 
        $paramManifesta["IDCIENCIAEMISS"] = (isset($paramManifesta["IDCIENCIAEMISS"])) ? $paramManifesta["IDCIENCIAEMISS"] : null;
        $idManifestacao = ($paramManifesta["IDCIENCIAEMISS"] == '4') ? 4 : (isset($paramManifesta["IDCONFIRMAEMISS"])) ? 1 : 0;
        $titulosSucesso = 0;
        foreach ($notas as $nota) {
            try {
                if($mode == 'B') {
                    // Rotina para verificação das contas a pagar já geradas (se estão baixadas, etc)
                    $this->dfe47000LiberacaoParaFinanceiroFactory->cancelaVencimento($nota["CDFILIAL"], $nota["NRLANCTONF"]);
                }
                
                if($mode == 'L') {
                    $data = date("d/m/Y H:i:s");
                    $this->dfe47000LiberacaoParaFinanceiroFactory->updateIdBloqueado('L', $this->environment->getCdOperador(), '', $data, $nota["CDFILIAL"], $nota["NRLANCTONF"]);
                    $this->dfe47000LiberacaoParaFinanceiroFactory->updateIdBloqueadoCP('L', $nota["CDFILIAL"], $nota["NRLANCTONF"]);
                    // Manifesta a nota de acordo com a parametrização
                    
                    if($idManifestacao != 0) {
                        try {
                            if(!isset($configs[$nota["CDEMPRESA"]])) {
                                $config     = $this->manifestacaoDestinatarioService->montaConfigManifestacao($nota["CDEMPRESA"], $this->environment->getNrOrgTrab(), $nota["CDINSCESTA"], '', true);
                                $configs[$nota["CDEMPRESA"]] = $config;
                            }
                            else {
                                $config = $configs[$nota["CDEMPRESA"]];
                            }
                            $retorno = $this->manifestacaoDestinatarioService->manifestarNfe(array($nota), $config, $idManifestacao);
                        }
                        catch(\Exception $e) {}
                    }
                }
                else {
                    $this->dfe47000LiberacaoParaFinanceiroFactory->updateIdBloqueado('B', $this->environment->getCdOperador(), '', '', $nota["CDFILIAL"], $nota["NRLANCTONF"]);
                    $this->dfe47000LiberacaoParaFinanceiroFactory->updateIdBloqueadoCP('B', $nota["CDFILIAL"], $nota["NRLANCTONF"]);
                    
                }
                $titulosSucesso++;
                
            }
            catch(\Exception $e) {
                array_push($this->erros, array("NRNOTAFISC"     => $nota["NRNOTAFISC"],
                                         "DTEMISSAO"            => $nota["DTEMISSAO"],
                                         "VRNOTAFISC"           => $nota["VRNOTAFISC"],
                                         "DSERRO"               => $e->getMessage()));
            }
        }
        return array("SUCESSOS" => $titulosSucesso, "ERROS" => $this->erros);
    }
    
    
    
    public function updateNotaFiscal($nota, $mode){
        $data = date("d/m/Y H:i:s");
        try {
            if($mode == 'L') {
                $this->dfe47000LiberacaoParaFinanceiroFactory->updateIdBloqueado('L', $this->environment->getCdOperador(), '', $data, $nota["CDFILIAL"], $nota["NRLANCTONF"]);
                $paramManifesta = $this->dfe47000LiberacaoParaFinanceiroFactory->getParamManifesta(); 
                $idManifestacao = ($paramManifesta["IDCIENCIAEMISS"] == '4') ? 4 : ($paramManifesta["IDCONFIRMAEMISS"]) ? 1 : 0;
                if($idManifestacao != 0) {
                    try {
                        if(!isset($configs[$nota["CDEMPRESA"]])) {
                            $config     = $this->manifestacaoDestinatarioService->montaConfigManifestacao($nota["CDEMPRESA"], $this->environment->getNrOrgTrab(), $nota["CDINSCESTA"], '', true);
                            $configs[$nota["CDEMPRESA"]] = $config;
                        }
                        else {
                            $config = $configs[$nota["CDEMPRESA"]];
                        }
                        $retorno = $this->manifestacaoDestinatarioService->manifestarNfe(array($nota), $config, $idManifestacao);
                    }
                    catch(\Exception $e) {}
                }
                
            }else{
                // Rotina para verificação das contas a pagar já geradas (se estão baixadas, etc)
               $this->dfe47000LiberacaoParaFinanceiroFactory->updateIdBloqueado('B', $this->environment->getCdOperador(), '', $data, $nota["CDFILIAL"], $nota["NRLANCTONF"]);
            }
        }catch(\Exception $e) {
            array_push($this->erros, array("NRNOTAFISC"     => $nota["NRNOTAFISC"],
                                     "DTEMISSAO"            => $nota["DTEMISSAO"],
                                     "VRNOTAFISC"           => $nota["VRNOTAFISC"],
                                     "DSERRO"               => $e->getMessage()));
        }
    }
    
    public function setAprovacaoCP($notas, $mode, $nrnotafisc, $dtEmissao){
        $titulosSucesso = 0;
        foreach($notas as $nota){
            try{
                
                if($mode == 'L'){
                    $this->dfe47000LiberacaoParaFinanceiroFactory->updateContaPagar('L', $nota["CDFILIAL"], $nota["NRLANCTONF"], $nota["NRPARCELAPAG"], $nota["NRSEQVENPAG"]);
                }else{
                    $this->dfe47000LiberacaoParaFinanceiroFactory->updateContaPagar('B', $nota["CDFILIAL"], $nota["NRLANCTONF"], $nota["NRPARCELAPAG"], $nota["NRSEQVENPAG"]);
                }
                $titulosSucesso++;
                
            }catch(\Exception $e) {
                array_push($this->erros, array("NRNOTAFISC"     => $nrnotafisc,
                                         "DTEMISSAO"            => $dtEmissao,
                                         "VRNOTAFISC"            => $nota["VRATUAPAG"],
                                         "DSERRO"               => $e->getMessage()));
            }
        }
        return array("SUCESSOS" => $titulosSucesso, "ERROS" => $this->erros);
    }
    
    public function getContas($cdFilial, $nrlanctonf){
        return $this->dfe47000LiberacaoParaFinanceiroFactory->getContas($cdFilial, $nrlanctonf); 
    }
    
    public function getIdBloqnffin($cdFilial, $nrlanctonf){
        return $this->dfe47000LiberacaoParaFinanceiroFactory->getIdBloqnffin($cdFilial, $nrlanctonf); 
    }
    
    public function updateJustificativa($notas){
        foreach($notas as $nota){
            $this->dfe47000LiberacaoParaFinanceiroFactory->updateJustificativa($nota["CDFILIAL"], $nota["NRLANCTONF"], $notas[0]["DSOBSRECUSA"]);
        }
        
    }
    
    protected function renderTemplate($template, $emitente) {
        $search = array(
            '{emitente}'
        );
        $replace = array(
          $emitente
        );
        $template = str_replace($search, $replace, $template);
        return $template;
    }
    
    public function enviaEmailRecusa($nota, $justificativa, $config){
        $aMails =  $this->dfe47000LiberacaoParaFinanceiroFactory->getEmailRecusa($nota["CDFILIAL"], $nota["NRLANCTONF"]);
        
        if(!empty($aMails)) {
        
            $template = '<p>Justificativa da recusa: '. $justificativa. '.</p>Atenciosamente,<br> {emitente}';
            if(!empty($aMails["DSEMAILOPER"])){
                $aMailsDest[0] = $aMails["DSEMAILOPER"];
                
            }
            
            if(!empty($aMails["EMAILAPROVA"]) && !empty($aMailsDest)){
                if($aMails["EMAILAPROVA"] != $aMails["DSEMAILOPER"]){
                    $aMailsDest[1] = $aMails["EMAILAPROVA"];
                }
            }else if(!empty($aMails["EMAILAPROVA"])){
                $aMailsDest[0] =  $aMails["EMAILAPROVA"];
            }
            
            $config = $this->recolocarArrobaEmail(json_encode($config));
            
            $config = json_decode($config);
            
            $template = $this->renderTemplate(
                $template, $config->aMailConf->fantasy
            );
            
            $this->smtpService = new SMTP(array(
                                            'host'          => $config->aMailConf->host,
                                            'username'      => $config->aMailConf->username,
                                            'password'      => $config->aMailConf->password,
                                            'port'          => $config->aMailConf->port,
                                            'smtpSecure'    => $config->aMailConf->secure
                                            ));
                                            
            $this->smtpService->setFrom($config->aMailConf->from);
            $this->smtpService->setFromName($config->aMailConf->fantasy);
            
            $this->smtpService->setSubject("Nota Fiscal recusada, Nota Fiscal: ".$nota["NRNOTAFISC"]);
            $enviado = $this->smtpService->sendMail($template, $aMailsDest);
    
            return $enviado;
        }
    }
    
    private function recolocarArrobaEmail($config)
    {
        $config            = json_decode($config);
        $config->aMailConf = json_decode(str_replace("&#64", "@", json_encode($config->aMailConf)));
        $config            = json_encode($config);
        return $config;
    }
    
}