<?php
namespace Teknisa\DF\Modules\Entrada\Services;

use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Doctrine\ORM\EntityManager;
use Teknisa\DF\Modules\Entrada\SincronizacaoNFSe\Utilities\Util;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Row;
use Zeedhi\Framework\Remote\cURLRequest;
use Zeedhi\Framework\Remote\HttpInterface;

class ImportacaoNFSeService extends \Teknisa\DF\Controllers\TeknisaCrud {
    const CAMINHO_MUNICIPIOS = 
        array(
              "7107" => "Teknisa\DF\Modules\Entrada\SincronizacaoNFSe\ConsultaNFSeSaoPaulo"
            );
            
    const FILESERVER_LINK   = 'http://midia.teknisa.com/files/';
    const MSG_ERROS = array(
            "SEM_DIR" => "Diretório parametrizado não existe",
            "DIR_INV" => "Não há diretório parametrizado. Favor indicar o diretório onde estão os arquivos a serem
                          importados na tela de Parâmetros de Integração.",
            "IMP_ASO" => "Impostos da nota fiscal eletrônica devem ser associados.",
            "ENC_ASO" => "Todos os encargos da nota fiscal eletrônica devem ser associados.",
            "NAO_NOT" => "Não há notas no diretório."
        );
            
    protected $environment;
    protected $connection;
    protected $newCode;
    protected $xmlDirectory;
    protected $certUploadDirectory;
    protected $aprovacaoLancamentoService;
    protected $entityManager;
    protected $importacaoConsultaNotaFiscalFactory; 
    protected $manifestacaoDestinatarioFactory;
    protected $SAAS = false;
    
    public function __construct( $environment, EntityManager $entityManager, $newCode,
                            $certUploadDir, $xmlDir, $fileServerLink, $fileUploader , $dfe51000ImportacaoConsultaNotaFiscalFactory, $manifestacaoDestinatarioFactory
                            ) {
        $this->environment = $environment;
        $this->entityManager = $entityManager;
        $this->connection = $entityManager->getConnection();
        $this->newCode = $newCode;
        $this->xmlDirectory = $xmlDir;
        $this->certUploadDirectory = $certUploadDir;
        $this->fileServerLink = $fileServerLink;
        $this->fileUploader = $fileUploader;
        $this->importacaoConsultaNotaFiscalFactory = $dfe51000ImportacaoConsultaNotaFiscalFactory;
        $this->manifestacaoDestinatarioFactory = $manifestacaoDestinatarioFactory;
        date_default_timezone_set("America/Sao_Paulo");
    }
    
    public function sincronizarNFSe($row){
        $nfsImportadas = 0;
        $cdempresa = $row["CDEMPRESA"]; 
        $cdinscmuni = $row["CDINSCMUNI"];
        $nrorg = $this->environment->getNrOrg();
        
        if($row["FORMAPESQUISA"] == 0){
            $data = Util::GetUltimoRegistro($this->connection, $cdempresa,$nrorg );
            $data = substr($data[0]["MAX(DTULTATU)"],0,10);
            if ($data != null){
                $row["DTINICIO"] = $this->dateEmMysql($data);
                $row["DTFIM"] = date("Y-m-d");
           
            }else{
                $row["DTINICIO"] = date('Y-m-d', strtotime('-3 month', strtotime(date("Y-m-d"))));
                $row["DTFIM"] = date("Y-m-d");
                
            }
        } else{
                $row["DTINICIO"] = date('Y-m-d', strtotime('-3 month', strtotime(date("Y-m-d"))));
                $row["DTFIM"] = date("Y-m-d");
     
        }
        $dataset = Util::getDadosGeracao($this->connection, $cdempresa, $cdinscmuni, $nrorg);
        if(!is_null($dataset)) {
            if(\Zeedhi\Framework\Util\Functions::arrayKeyExists($dataset["CDMUNICIPIO"], self::CAMINHO_MUNICIPIOS)) {
                $caminho = self::CAMINHO_MUNICIPIOS[$dataset["CDMUNICIPIO"]];
                $localServidor = Util::getLocalServidor($this->connection,$cdempresa, $cdinscmuni, $nrorg);
                    if ($localServidor == 1) { 
                        $nfse = new $caminho($this->importacaoConsultaNotaFiscalFactory, $row, $this->connection, $this->environment, $this->newCode, $this->xmlDirectory, $this->certUploadDirectory);
                    }else if ($localServidor == 2) {
                        $arrayCertificado = $this->getLinkCertificado($cdempresa, $cdinscmuni, $nrorg);
                        $nfse = new $caminho($this->importacaoConsultaNotaFiscalFactory, $row, $this->connection, $this->environment, $this->newCode, $this->xmlDirectory, null, $arrayCertificado["CERTIFICADO"], $arrayCertificado['CERTKEY']);
                    }else if  ($localServidor == null) {
                        throw new \Exception("Local do servidor dos parâmetros de NFS-e não foi encontrado");
                    }
                $notas = $nfse->getNotas();
                $this->salvaXmlNFSe($notas , $localServidor);
                return $this->importarNotas($nfse);
                
            }else{
                throw new \Exception("Funções de Importação de NFS-e não habilitadas para esse município.");
            }
        }
    }
    
    private function getLinkCertificado($cdempresa, $cdinscmuni, $nrorg) {
        $certKey = null;
        $certificado = null;
        $certificados = Util::getDataSetCertificadoFileServer($this->connection, $cdinscmuni, $cdempresa, $nrorg, $this->fileServerLink);

        foreach($certificados as $cert) {
            $extension = explode(".", $cert['fileName']);
            $extension = end($extension);
            if($extension === 'pfx' || $extension === 'p12') $certificado  = $cert['path'];
            else if($extension === 'pem') $certKey = $cert['path'];
        }
        if(is_null($certificado)) throw new \Exception('Certificado não encontrado.');
        else if(is_null($certKey))     throw new \Exception('CertKey não encontrado.');
        else {
            return array("CERTIFICADO" => $certificado, "CERTKEY" => $certKey);
        }
    }

    public function salvaXmlNFSe($notas, $localServidor){
        if(!is_null($notas)){
            foreach($notas as $nota){
                $chave = $nota->ChaveNFe->NumeroNFe;
                 $json = json_encode($chave);
                $chave = json_decode($json,TRUE)[0];
                if(!isset(Util::existeNotaRelac($chave, $this->connection)[0])){
                    $this->manifestacaoDestinatarioFactory->zGravaFile($localServidor, $chave.'.xml', $nota->asXML(),
                    			                                    'importar/', null, null, $chave);
                }
            }   
        }
    }

    protected function converteXmlParaArray($stringXml) {
        try{
            $xml = simplexml_load_string($stringXml);
            $json = json_encode($xml);
            $array = json_decode($json,TRUE);
            return $array;
        }
        catch(\Exception $e){
            throw new \Exception(self::ERROS_IMPORTACAO["ARQ_INV"]);
        }
    }
    
    public function importarNotas($nfse) {
        try {
            $nomeArquivos = $this->buscaNomeArquivosNotasImportacao();
            $impostos = array( "PIS" => '', "CSLL" => '', "IR" => '',
                              "COFINS" => '', "INSS" => '', "PIS" => '',
                              "ISS" => '')
                              ; 
                              
            $this->verificaAssociaImpostos($impostos);
            
            $encargos = array();
            
            $paramIntegracao = $this->importacaoConsultaNotaFiscalFactory->getParamIntegracao();
            
            $contImp = 0;
            $contErro = 0;
            
            foreach($nomeArquivos as $nomeArquivo) {
                $arquivo = file_get_contents($nomeArquivo);
                try {
                    $this->connection->beginTransaction();
                    $xmlNotaFiscal  = new \DOMDocument("1.0", "UTF-8");
                    try{
            		    $xmlNotaFiscal->loadXML($arquivo, LIBXML_NOBLANKS);
                    }
                    catch(\Exception $e){
                        throw new \Exception(self::ERROS_IMPORTACAO["ARQ_INV"]);
                    }
        		     
                    $arrayXml  = $this->converteXmlParaArray($arquivo);

                    $nfse->importacaoXML($nomeArquivo, $arrayXml, $arquivo, $this->environment->getCdOperador(), $contImp, $contErro, $impostos, $encargos, $paramIntegracao);
    
                    $this->connection->commit();
                }
                catch(\Exception $e) {
                    $this->connection->rollback();
                    $contErro++;
                    return $e->getMessage();
                }
            }
        }
        catch(\Exception $e) {
            throw $e;
        }
        
        return array("ERROS" => $contErro, "SUCESSOS" => $contImp);
    }
    
       private function buscaNomeArquivosNotasImportacao() {
        if($this->SAAS = false) {
            $diretorio = $this->importacaoConsultaNotaFiscalFactory->getDiretorio();   
            
            if(isset($diretorio[0])) {
                $diretorio = $diretorio[0]["DSDIRIMPARQXML"];
                $tam = strlen($diretorio);
                if(substr($diretorio, $tam-1, $tam) !== "/")
                    $diretorio = $diretorio."/";
                
                
                $this->diretorioImportacao = $diretorio;
                $this->criaDiretorios();
                
                $arquivos  = $this->getNomeArquivosDiretorio($diretorio);
                if($arquivos)  return $arquivos; 
                else throw new \Exception(self::MSG_ERROS["NAO_NOT"]);
            }
            else {
                throw new \Exception(self::MSG_ERROS["SEM_DIR"]);
            }
        }
        else {
            $notas = $this->importacaoConsultaNotaFiscalFactory->existeNotaRelac();
            $arquivos = array();
            if(isset($notas[0])) {
                foreach($notas as $nota) {
                    array_push($arquivos, self::FILESERVER_LINK.$nota["FILESERVERLINK"]); 
                }
            }
            else throw new \Exception(self::MSG_ERROS["NAO_NOT"]);
        }
        
        return $arquivos;
    }
    
    /**
    *   Busca o nome de todos os arquivos de um diretório.
    *   
    *   @param string $diretorio - Diretório de onde os arquivos serão buscados 
    *    
    *   return array - Array com nome dos arquivos
    */
    
    private function getNomeArquivosDiretorio($diretorio) {
        if(is_dir($diretorio)) {
            return glob("$diretorio{*.xml,*.XML}", GLOB_BRACE);
        }
        else {
            throw new \Exception(self::MSG_ERROS["DIR_INV"]);
        }
    }
    
    private function verificaAssociaImpostos(&$impostos) {
        $buscaImpostos = $this->importacaoConsultaNotaFiscalFactory->getImpostosMFSe();
        if(isset($buscaImpostos[0])) {
            foreach ($buscaImpostos as $imposto) {
                switch ($imposto["SGIMPOSTONFE"]) {
                    case 'ICMS':
                        $impostos["ICMS"]       = $imposto["CDIMPOSTO"];
                        break;
                    case 'IPI':
                        $impostos["IPI"]        = $imposto["CDIMPOSTO"];
                        break;
                    case 'ISSQN-E':
                        $impostos["ISSQN"]      = $imposto["CDIMPOSTO"];
                        break;
                    case 'PIS':
                        $impostos["PIS"]        = $imposto["CDIMPOSTO"];
                        break;
                    case 'COFINS':
                        $impostos["COFINS"]     = $imposto["CDIMPOSTO"];
                        break;
                    case 'II':
                        $impostos["II"]         = $imposto["CDIMPOSTO"];
                        break;
                    case 'ICMS ST':
                        $impostos["STICMS"]     = $imposto["CDIMPOSTO"];
                        break;
                    case 'PIS ST':
                        $impostos["STPIS"]      = $imposto["CDIMPOSTO"];
                        break;
                    case 'COFINS ST':
                        $impostos["STCOFINS"]   = $imposto["CDIMPOSTO"];
                        break;
                    case 'INSS':
                        $impostos["INSS"]       = $imposto["CDIMPOSTO"];
                        break;
                    case 'ISS':
                        $impostos["ISS"]        = $imposto["CDIMPOSTO"];
                        break;
                    case 'IR':
                        $impostos["IR"]         = $imposto["CDIMPOSTO"];
                        break;
                    case 'CSLL':
                        $impostos["CSLL"]       = $imposto["CDIMPOSTO"];
                        break;
                }
            }
            /*foreach ($impostos as $nomeImposto => $cdImposto) {
                if(empty($cdImposto)) throw new \Exception(self::MSG_ERROS["IMP_ASO"]." Imposto $nomeImposto não associado.");
            }*/
        }
        else throw new \Exception(self::MSG_ERROS["IMP_ASO"]);
    }
    
    public static function dateEmMysql($dateSql){
        $ano= substr($dateSql, 6);
        $mes= substr($dateSql, 3,-5);
        $dia= substr($dateSql, 0,-8);
        return $ano."-".$mes."-".$dia;
    }
    
     private function criaDiretorios() {
        $diretorios = array(
            "CTe" => $this->diretorioImportacao.DIRECTORY_SEPARATOR.'CTe',
            "CTe/IMPORTADOS" => $this->diretorioImportacao.DIRECTORY_SEPARATOR.'CTe'.DIRECTORY_SEPARATOR.'importados',
            "CTe/INVALIDOS"  => $this->diretorioImportacao.DIRECTORY_SEPARATOR.'CTe'.DIRECTORY_SEPARATOR.'invalidos',
            "CTe/DUPLICADOS" => $this->diretorioImportacao.DIRECTORY_SEPARATOR.'CTe'.DIRECTORY_SEPARATOR.'duplicados',
            "CTe/BLOQUEADOS" => $this->diretorioImportacao.DIRECTORY_SEPARATOR.'CTe'.DIRECTORY_SEPARATOR.'bloqueados',
            "CTe/BLOQUEADOSFORN" => $this->diretorioImportacao.DIRECTORY_SEPARATOR.'CTe'.DIRECTORY_SEPARATOR.'bloqueados/fornecedores',
            "IMPORTADOS" => $this->diretorioImportacao.DIRECTORY_SEPARATOR.'importados',
            "INVALIDOS"  => $this->diretorioImportacao.DIRECTORY_SEPARATOR.'invalidos',
            "DUPLICADOS" => $this->diretorioImportacao.DIRECTORY_SEPARATOR.'duplicados',
            "BLOQUEADOS" => $this->diretorioImportacao.DIRECTORY_SEPARATOR.'bloqueados',
            "BLOQUEADOSFORN" => $this->diretorioImportacao.DIRECTORY_SEPARATOR.'bloqueados/fornecedores'
        );

        foreach($diretorios as $diretorio) {
            if (!is_dir($diretorio)) {
                if (!mkdir($diretorio, 0777, true)) {
                    throw new \Exception(
                        "Não foi possivel criar o diretorio $diretorio. Verifique as permissões"
                    );
                }
                chmod($diretorio, 0777);
            }
        }
    }

 
}

?>