<?php

namespace Teknisa\DF\Modules\Saida\Factories;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\DF\Modules\Saida\Factories\ManifestoDocumentoEletronicoService;

use Teknisa\Libs\Util\Environment;
    
class ManifestoDocumentoEletronicoFactory {
    
    private $entityManager;
    private $connection;

    public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
        $this->connection = $entityManager->getConnection();
    }
    
    public function getDefaultMdfe($cdEmpresa) {
        $sql = "SELECT IE.CDEMPRESA,
             IE.CDINSCESTA, 
             IE.IDTPIJURIEST, 
             IE.NRINSJURIEST,
             IE.CDPAIS, 
             IE.SGESTADO, 
             IE.CDMUNICIPIO, 
             IE.DSENDEIEST, 
             IE.NRCEPIEST,
             IE.NRTELEIEST,
             IE.CDBAIRRO,
             IE.DSCOMPENDIES, 
             IE.NRSERIECERTIFIC, 
             IE.IDTPIMPRESDAMDFE, 
             IE.IDTPAMBIENTEMDFE,
             IE.CDSERIEMDFE, 
             IE.CDMODDOCFISC AS CDMODDOCFISCMDFE,
             EP.NMRAZSOCEMPR,
             EP.NMEMPRESA,
             IE.IDPRESTSERVTRANSP,
             'P' AS IDSTATUSMDFE,
             'Pendente' AS DSSTATUSMDFE,
             '0' AS IDTIPOEMISMDFE,
             'P' AS IDEMAILMDFE,
             0 AS VRCARGAMDFE,
             SYSDATE AS DTMDFE
        FROM INSCRESTAD IE JOIN EMPRESA EP ON (EP.CDEMPRESA = IE.CDEMPRESA) 
       WHERE EP.CDEMPRESA = :P_CDEMPRESA";
       
       $params = array(
            "P_CDEMPRESA" => $cdEmpresa 
        );
        
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        return $query->fetch();
    }
    
    public function insereCarregamentoMdfe($cdviatura, $nrlanctomdfe, $cdpais, $sgestado, $cdmunicipio) {
        $sql = "INSERT INTO PERCURSOMDFE (CDVIATURA, NRLANCTOMDFE, NRSEQPERCURSO, NRORDEMENTREGA, CDPAIS, SGESTADO, CDMUNICIPIO) VALUES (:P_CDVIATURA, :P_NRLANCTOMDFE, '0001', '000', :P_CDPAIS, :P_SGESTADO, :P_CDMUNICIPIO)";
    
        $params = array(
            "P_CDVIATURA" => $cdviatura,
            "P_NRLANCTOMDFE" => $nrlanctomdfe,
            "P_CDPAIS" => $cdpais,
            "P_SGESTADO" => $sgestado,
            "P_CDMUNICIPIO" => $cdmunicipio
        );
        
        $query = $this->connection->prepare($sql);
        return $query->execute($params);
    }
    
    public function getSequencialSeguroCarga($cdviatura, $nrlanctomdfe) {
        $sql = "SELECT NVL(MAX(S.NRSEQSEGCARMDFE), 0) + 1 AS CODIGO FROM SEGUROCARGAMDFE S WHERE S.CDVIATURA = :P_CDVIATURA AND S.NRLANCTOMDFE = :P_NRLANCTOMDFE";
       
       $params = array(
            "P_CDVIATURA" => $cdviatura,
            "P_NRLANCTOMDFE" => $nrlanctomdfe
        );
        
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        return $query->fetch()["CODIGO"];
    }
    
    public function getSequencialAverbacao($cdviatura, $nrlanctomdfe, $nrseqsegcarmdfe) {
        $sql = "SELECT NVL(MAX(A.NRSEQAVERBAMDFE), 0) + 1 AS CODIGO FROM AVERBACAOMDFE A WHERE A.CDVIATURA = :P_CDVIATURA AND A.NRLANCTOMDFE = :P_NRLANCTOMDFE AND A.NRSEQSEGCARMDFE = :P_NRSEQSEGCARMDFE";
       
        $params = array(
            "P_CDVIATURA" => $cdviatura,
            "P_NRLANCTOMDFE" => $nrlanctomdfe,
            "P_NRSEQSEGCARMDFE" => $nrseqsegcarmdfe
        );
        
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        return $query->fetch()["CODIGO"];
    }
    
        
    public function getSequencialDadosAdic($cdviatura, $nrlanctomdfe) {
        $sql = "SELECT NVL(MAX(D.NRSEQDADOSADIC), 0) + 1 AS CODIGO FROM DADOSADICMDFE D WHERE D.CDVIATURA = :P_CDVIATURA AND D.NRLANCTOMDFE = :P_NRLANCTOMDFE";
       
        $params = array(
            "P_CDVIATURA" => $cdviatura,
            "P_NRLANCTOMDFE" => $nrlanctomdfe
        );
        
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        return $query->fetch()["CODIGO"];
    }
    
    
    public function getSequencialNotasRelac($cdviatura, $nrlanctomdfe) {
        $sql = "SELECT NVL(MAX(TO_NUMBER(N.NRSEQRELACNF)), 0) + 1 AS NRSEQRELACNF, NVL(MAX(TO_NUMBER(N.NRORDEMENTREGA)), 0) + 1 AS NRORDEMENTREGA FROM NFRELACMDFE N WHERE N.CDVIATURA = :P_CDVIATURA AND N.NRLANCTOMDFE = :P_NRLANCTOMDFE";
       
        $params = array(
            "P_CDVIATURA" => $cdviatura,
            "P_NRLANCTOMDFE" => $nrlanctomdfe
        );
        
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        return $query->fetch();
    }
    
    public function getSequencialPercurso($cdviatura, $nrlanctomdfe) {
        $sql = "SELECT NVL(MAX(TO_NUMBER(P.NRSEQPERCURSO)), 0) + 1 AS NRSEQPERCURSO, NVL(MAX(TO_NUMBER(P.NRORDEMENTREGA)), -1) + 1 AS NRORDEMENTREGA FROM PERCURSOMDFE P WHERE P.CDVIATURA = :P_CDVIATURA AND P.NRLANCTOMDFE = :P_NRLANCTOMDFE";
       
        $params = array(
            "P_CDVIATURA" => $cdviatura,
            "P_NRLANCTOMDFE" => $nrlanctomdfe
        );
        
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        return $query->fetch();
    }
    
    public function limparMotorista($cdviatura, $nrlanctomdfe) {
        $sql = "DELETE FROM MOTORISTAMDFE WHERE CDVIATURA = :P_CDVIATURA AND NRLANCTOMDFE = :P_NRLANCTOMDFE";
        
        $params = array(
            "P_CDVIATURA" => $cdviatura,
            "P_NRLANCTOMDFE" => $nrlanctomdfe
        );
        
        $query = $this->connection->prepare($sql);
        return $query->execute($params);
    }
    
    public function limparReboque($cdviatura, $nrlanctomdfe) {
        $sql = "DELETE FROM REBOQUEMDFE WHERE CDVIATURA = :P_CDVIATURA AND NRLANCTOMDFE = :P_NRLANCTOMDFE";
        
        $params = array(
            "P_CDVIATURA" => $cdviatura,
            "P_NRLANCTOMDFE" => $nrlanctomdfe
        );
        
        $query = $this->connection->prepare($sql);
        return $query->execute($params);
    }
    
    public function limparAverbacao($cdviatura, $nrlanctomdfe) {
        $sql = "DELETE FROM AVERBACAOMDFE WHERE CDVIATURA = :P_CDVIATURA AND NRLANCTOMDFE = :P_NRLANCTOMDFE";
        
        $params = array(
            "P_CDVIATURA" => $cdviatura,
            "P_NRLANCTOMDFE" => $nrlanctomdfe
        );
        
        $query = $this->connection->prepare($sql);
        return $query->execute($params);
    }
    
    public function limparSeguroCarga($cdviatura, $nrlanctomdfe) {
        $sql = "DELETE FROM SEGUROCARGAMDFE WHERE CDVIATURA = :P_CDVIATURA AND NRLANCTOMDFE = :P_NRLANCTOMDFE";
        
        $params = array(
            "P_CDVIATURA" => $cdviatura,
            "P_NRLANCTOMDFE" => $nrlanctomdfe
        );
        
        $query = $this->connection->prepare($sql);
        return $query->execute($params);
    }
    
    public function limparDadosAdic($cdviatura, $nrlanctomdfe) {
        $sql = "DELETE FROM DADOSADICMDFE WHERE CDVIATURA = :P_CDVIATURA AND NRLANCTOMDFE = :P_NRLANCTOMDFE";
        
        $params = array(
            "P_CDVIATURA" => $cdviatura,
            "P_NRLANCTOMDFE" => $nrlanctomdfe
        );
        
        $query = $this->connection->prepare($sql);
        return $query->execute($params);
    }
    
    public function mudarOrdemNota($cdviatura, $nrlanctomdfe, $nrseqrelacnf, $diminuirOrdem) {
        if ($diminuirOrdem) {
            $sql = "UPDATE NFRELACMDFE SET NRORDEMENTREGA = NRORDEMENTREGA + 1 WHERE CDVIATURA = :P_CDVIATURA AND NRLANCTOMDFE = :P_NRLANCTOMDFE AND NRORDEMENTREGA = ((SELECT NRORDEMENTREGA FROM NFRELACMDFE N WHERE N.CDVIATURA = :P_CDVIATURA AND N.NRLANCTOMDFE = :P_NRLANCTOMDFE AND N.NRSEQRELACNF = :P_NRSEQRELACNF) - 1)";
        }
        else {
            $sql = "UPDATE NFRELACMDFE SET NRORDEMENTREGA = NRORDEMENTREGA - 1 WHERE CDVIATURA = :P_CDVIATURA AND NRLANCTOMDFE = :P_NRLANCTOMDFE AND NRORDEMENTREGA = ((SELECT NRORDEMENTREGA FROM NFRELACMDFE N WHERE N.CDVIATURA = :P_CDVIATURA AND N.NRLANCTOMDFE = :P_NRLANCTOMDFE AND N.NRSEQRELACNF = :P_NRSEQRELACNF) + 1)";
        }

        $params = array(
            "P_CDVIATURA" => $cdviatura,
            "P_NRLANCTOMDFE" => $nrlanctomdfe,
            "P_NRSEQRELACNF" => $nrseqrelacnf
        );
        
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        
        if ($diminuirOrdem) {
            $sql = "UPDATE NFRELACMDFE SET NRORDEMENTREGA = NRORDEMENTREGA - 1 WHERE CDVIATURA = :P_CDVIATURA AND NRLANCTOMDFE = :P_NRLANCTOMDFE AND NRSEQRELACNF = :P_NRSEQRELACNF";
        }
        else {
            $sql = "UPDATE NFRELACMDFE SET NRORDEMENTREGA = NRORDEMENTREGA + 1 WHERE CDVIATURA = :P_CDVIATURA AND NRLANCTOMDFE = :P_NRLANCTOMDFE AND NRSEQRELACNF = :P_NRSEQRELACNF";
        }
        
        $query = $this->connection->prepare($sql);
        return $query->execute($params);
    }
    
    public function excluirNota($cdviatura, $nrlanctomdfe, $nrseqrelacnf) {
        $sql = "UPDATE NFRELACMDFE SET NRORDEMENTREGA = NRORDEMENTREGA - 1 WHERE CDVIATURA = :P_CDVIATURA AND NRLANCTOMDFE = :P_NRLANCTOMDFE AND NRORDEMENTREGA > ((SELECT NRORDEMENTREGA FROM NFRELACMDFE N WHERE N.CDVIATURA = :P_CDVIATURA AND N.NRLANCTOMDFE = :P_NRLANCTOMDFE AND N.NRSEQRELACNF = :P_NRSEQRELACNF) - 1)";

        $params = array(
            "P_CDVIATURA" => $cdviatura,
            "P_NRLANCTOMDFE" => $nrlanctomdfe,
            "P_NRSEQRELACNF" => $nrseqrelacnf
        );
        
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        
        $sql = "DELETE FROM NFRELACMDFE WHERE CDVIATURA = :P_CDVIATURA AND NRLANCTOMDFE = :P_NRLANCTOMDFE AND NRSEQRELACNF = :P_NRSEQRELACNF";
        
        $query = $this->connection->prepare($sql);
        return $query->execute($params);
    }
    
    public function limparNota($cdviatura, $nrlanctomdfe) {
        $sql = "DELETE FROM NFRELACMDFE WHERE CDVIATURA = :P_CDVIATURA AND NRLANCTOMDFE = :P_NRLANCTOMDFE";
        
        $params = array(
            "P_CDVIATURA" => $cdviatura,
            "P_NRLANCTOMDFE" => $nrlanctomdfe
        );
        
        $query = $this->connection->prepare($sql);
        return $query->execute($params);
    }
    
    public function excluiMDFE($cdviatura, $nrlanctomdfe){
        $sql = "DELETE FROM MDFE WHERE CDVIATURA = :P_CDVIATURA AND NRLANCTOMDFE = :P_NRLANCTOMDFE";
        
        $params = array(
            "P_CDVIATURA" => $cdviatura,
            "P_NRLANCTOMDFE" => $nrlanctomdfe
        );
       
        
        $query = $this->connection->prepare($sql);
        $query = $query->execute($params);
        
        return $query;
    }
    
    public function mudarOrdemPercurso($cdviatura, $nrlanctomdfe, $nrseqpercurso, $diminuirOrdem) {
        if ($diminuirOrdem) {
            $sql = "UPDATE PERCURSOMDFE SET NRORDEMENTREGA = NRORDEMENTREGA + 1 WHERE CDVIATURA = :P_CDVIATURA AND NRLANCTOMDFE = :P_NRLANCTOMDFE AND NRORDEMENTREGA = ((SELECT NRORDEMENTREGA FROM PERCURSOMDFE N WHERE N.CDVIATURA = :P_CDVIATURA AND N.NRLANCTOMDFE = :P_NRLANCTOMDFE AND N.NRSEQPERCURSO = :P_NRSEQPERCURSO) - 1)";
        }
        else {
            $sql = "UPDATE PERCURSOMDFE SET NRORDEMENTREGA = NRORDEMENTREGA - 1 WHERE CDVIATURA = :P_CDVIATURA AND NRLANCTOMDFE = :P_NRLANCTOMDFE AND NRORDEMENTREGA = ((SELECT NRORDEMENTREGA FROM PERCURSOMDFE N WHERE N.CDVIATURA = :P_CDVIATURA AND N.NRLANCTOMDFE = :P_NRLANCTOMDFE AND N.NRSEQPERCURSO = :P_NRSEQPERCURSO) + 1)";
        }

        $params = array(
            "P_CDVIATURA" => $cdviatura,
            "P_NRLANCTOMDFE" => $nrlanctomdfe,
            "P_NRSEQPERCURSO" => $nrseqpercurso
        );
        
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        
        if ($diminuirOrdem) {
            $sql = "UPDATE PERCURSOMDFE SET NRORDEMENTREGA = NRORDEMENTREGA - 1 WHERE CDVIATURA = :P_CDVIATURA AND NRLANCTOMDFE = :P_NRLANCTOMDFE AND NRSEQPERCURSO = :P_NRSEQPERCURSO";
        }
        else {
            $sql = "UPDATE PERCURSOMDFE SET NRORDEMENTREGA = NRORDEMENTREGA + 1 WHERE CDVIATURA = :P_CDVIATURA AND NRLANCTOMDFE = :P_NRLANCTOMDFE AND NRSEQPERCURSO = :P_NRSEQPERCURSO";
        }
        
        $query = $this->connection->prepare($sql);
        return $query->execute($params);
    }
    
    public function excluirPercurso($cdviatura, $nrlanctomdfe, $nrseqpercurso) {
        $sql = "UPDATE PERCURSOMDFE SET NRORDEMENTREGA = NRORDEMENTREGA - 1 WHERE CDVIATURA = :P_CDVIATURA AND NRLANCTOMDFE = :P_NRLANCTOMDFE AND NRORDEMENTREGA > ((SELECT NRORDEMENTREGA FROM PERCURSOMDFE N WHERE N.CDVIATURA = :P_CDVIATURA AND N.NRLANCTOMDFE = :P_NRLANCTOMDFE AND N.NRSEQPERCURSO = :P_NRSEQPERCURSO) - 1)";

        $params = array(
            "P_CDVIATURA" => $cdviatura,
            "P_NRLANCTOMDFE" => $nrlanctomdfe,
            "P_NRSEQPERCURSO" => $nrseqpercurso
        );
        
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        
        $sql = "DELETE FROM PERCURSOMDFE WHERE CDVIATURA = :P_CDVIATURA AND NRLANCTOMDFE = :P_NRLANCTOMDFE AND NRSEQPERCURSO = :P_NRSEQPERCURSO";
        
        $query = $this->connection->prepare($sql);
        return $query->execute($params);
    }
    
    public function limparPercurso($cdviatura, $nrlanctomdfe, $limparOrigem = false) {
        if ($limparOrigem) {
            $sql = "DELETE FROM PERCURSOMDFE WHERE CDVIATURA = :P_CDVIATURA AND NRLANCTOMDFE = :P_NRLANCTOMDFE";
        }
        else {
            $sql = "DELETE FROM PERCURSOMDFE WHERE CDVIATURA = :P_CDVIATURA AND NRLANCTOMDFE = :P_NRLANCTOMDFE AND TO_NUMBER(NRORDEMENTREGA) <> 0";
        }
        
        $params = array(
            "P_CDVIATURA" => $cdviatura,
            "P_NRLANCTOMDFE" => $nrlanctomdfe
        );
        
        $query = $this->connection->prepare($sql);
        return $query->execute($params);
    }
    
    public function atualizaDadosNotas($cdviatura, $nrlanctomdfe) {
        $sql = "SELECT SUM(NVL(NF.VRPBRUTOVOLU, 0)) AS VRPESOBRUTOCARGA, SUM(NVL(NF.VRNOTAFISC, 0)) AS VRCARGAMDFE FROM NOTAFISCAL NF JOIN NFRELACMDFE N ON N.CDVIATURA = :P_CDVIATURA AND N.NRLANCTOMDFE = :P_NRLANCTOMDFE WHERE NF.NRLANCTONF = N.NRLANCTONF AND NF.CDFILIAL = N.CDFILIAL";

        $params = array(
            "P_CDVIATURA" => $cdviatura,
            "P_NRLANCTOMDFE" => $nrlanctomdfe
        );
        
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        
        $dados = $query->fetch();
        $vrpesobrutocarga = $dados["VRPESOBRUTOCARGA"];
        $vrcargamdfe = $dados["VRCARGAMDFE"];
        
        $sql = "UPDATE MDFE SET VRPESOBRUTOCARGA = :P_VRPESOBRUTOCARGA, VRCARGAMDFE = :P_VRCARGAMDFE WHERE MDFE.CDVIATURA = :P_CDVIATURA AND MDFE.NRLANCTOMDFE = :P_NRLANCTOMDFE";
        
        $params = array(
            "P_CDVIATURA" => $cdviatura,
            "P_NRLANCTOMDFE" => $nrlanctomdfe,
            "P_VRPESOBRUTOCARGA" => $vrpesobrutocarga,
            "P_VRCARGAMDFE" => $vrcargamdfe
        );
        
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        
        return $dados;
    }
    
    public function setSequencialNrmdfe($cdviatura, $nrlanctomdfe, $nrinsjuriest) {
        $sql = "SELECT NVL(MAX(TO_NUMBER(NRMDFE)), 0) + 1 AS NRMDFE FROM MDFE WHERE NRINSJURIEST = :P_NRINSJURIEST";

        $params = array(
            "P_NRINSJURIEST" => $nrinsjuriest
        );
        
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        $nrmdfe = str_pad($query->fetch()["NRMDFE"], 9, "0", STR_PAD_LEFT);
        
        $sql = "UPDATE MDFE SET NRMDFE = :P_NRMDFE WHERE CDVIATURA = :P_CDVIATURA AND NRLANCTOMDFE = :P_NRLANCTOMDFE";
        
        $params = array(
            "P_CDVIATURA" => $cdviatura,
            "P_NRLANCTOMDFE" => $nrlanctomdfe,
            "P_NRMDFE" => $nrmdfe
        );
        
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        
        return $nrmdfe;
    }
    
    public function verificaPercurso ($cdviatura, $nrlanctomdfe, $sgestado, $cdmunicipio) {
        $sql = "SELECT NRSEQPERCURSO FROM PERCURSOMDFE WHERE CDVIATURA = :P_CDVIATURA AND NRLANCTOMDFE = :P_NRLANCTOMDFE AND SGESTADO = :P_SGESTADO AND CDMUNICIPIO = :P_CDMUNICIPIO";

        $params = array(
            "P_CDVIATURA" => $cdviatura,
            "P_NRLANCTOMDFE" => $nrlanctomdfe,
            "P_SGESTADO" => $sgestado,
            "P_CDMUNICIPIO" => $cdmunicipio
        );
        
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        $percurso = $query->fetch();
        if ($percurso) {
            return true; //Percurso já existe
        }
        
        return false; //Percurso não existe
    }
    
    public function setChaveAcessoMdfe($cdviatura, $nrlanctomdfe, $chaveAcesso) {
        $sql = "UPDATE MDFE SET NRACESSOMDFE = :P_NRACESSOMDFE WHERE CDVIATURA = :P_CDVIATURA AND NRLANCTOMDFE = :P_NRLANCTOMDFE";
        
        $params = array(
            "P_CDVIATURA" => $cdviatura,
            "P_NRLANCTOMDFE" => $nrlanctomdfe,
            "P_NRACESSOMDFE" => $chaveAcesso
        );
        
        $query = $this->connection->prepare($sql);
        return $query->execute($params);
    }
    
    public function setStatusMdfe($cdviatura, $nrlanctomdfe, $status) {
        $sql = "UPDATE MDFE SET IDSTATUSMDFE = :P_IDSTATUSMDFE WHERE CDVIATURA = :P_CDVIATURA AND NRLANCTOMDFE = :P_NRLANCTOMDFE";
        
        $params = array(
            "P_CDVIATURA" => $cdviatura,
            "P_NRLANCTOMDFE" => $nrlanctomdfe,
            "P_IDSTATUSMDFE" => $status
        );
        
        $query = $this->connection->prepare($sql);
        return $query->execute($params);
    }
    
    public function salvaXml($cdviatura, $nrlanctomdfe, $idtipoarqxmlmdfe, $xml) {
        $sql = "SELECT NRLANCTOMDFE FROM ARQXMLMDFE WHERE CDVIATURA = :P_CDVIATURA AND NRLANCTOMDFE = :P_NRLANCTOMDFE AND IDTIPOARQXMLMDFE = :P_IDTIPOARQXMLMDFE";

         $params = array(
            "P_CDVIATURA" => $cdviatura,
            "P_NRLANCTOMDFE" => $nrlanctomdfe,
            "P_IDTIPOARQXMLMDFE" => $idtipoarqxmlmdfe
        );
        
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        
        if($query->fetch()) {
            $sql = "UPDATE ARQXMLMDFE SET DSARQXMLMDFE = utl_raw.cast_to_raw(:P_DSARQXMLMDFE) WHERE CDVIATURA = :P_CDVIATURA AND NRLANCTOMDFE = :P_NRLANCTOMDFE AND IDTIPOARQXMLMDFE = :P_IDTIPOARQXMLMDFE";
        }
        else {
            $sql = "INSERT INTO ARQXMLMDFE (CDVIATURA, NRLANCTOMDFE, IDTIPOARQXMLMDFE, DSARQXMLMDFE) VALUES (:P_CDVIATURA, :P_NRLANCTOMDFE, :P_IDTIPOARQXMLMDFE, utl_raw.cast_to_raw(:P_DSARQXMLMDFE))";
            
        }
        
        $params = array(
            "P_CDVIATURA" => $cdviatura,
            "P_NRLANCTOMDFE" => $nrlanctomdfe,
            "P_IDTIPOARQXMLMDFE" => $idtipoarqxmlmdfe,
            "P_DSARQXMLMDFE" => $xml
        );
        
        $query = $this->connection->prepare($sql);
        return $query->execute($params);
    }
}
