<?php

namespace Teknisa\DF\Modules\Saida\Services;
use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\SMTP;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

use Teknisa\DF\Modules\Saida\Factories\Dfs61000TransmissaoNfseFactory;
use Geral\Util\DateUtil;


class Dfs61000TransmissaoNfseService {
    protected $transmissaoFactory;
    
    public function __construct(Dfs61000TransmissaoNfseFactory $transmissaoFactory) {
        $this->transmissaoFactory = $transmissaoFactory;
    }
    
    public function getCdReduCont($cdempresa, $cdcliente, $cdfilial, $nrlanctonf, $nrorg) {
        $cdreducont = $this->transmissaoFactory->getCdReduCont($cdempresa, $cdcliente, $nrorg);
        if(isset($cdreducont[0])) {
            if($cdreducont[0]["CDREDUCONT"]) return $cdreducont[0]["CDREDUCONT"];
        }
        $cdreducont = $this->transmissaoFactory->getCdReduCont1($cdfilial, $nrlanctonf, $nrorg);
        if(isset($cdreducont[0])) {
            if($cdreducont[0]["CDREDUCONT"]) return $cdreducont[0]["CDREDUCONT"];
        }
        $cdreducont = $this->transmissaoFactory->getCdReduCont2($cdempresa, $nrorg);
        return ($cdreducont[0]["CDREDUCONT"]) ? $cdreducont[0]["CDREDUCONT"] : NULL ;
    }
    
    public function getCarteira($cdempresa, $cdreducont, $nrorg) {
        $carteira = $this->transmissaoFactory->getCarteira($cdempresa, $cdreducont, $nrorg);
        return $carteira;
    }
    
    public function getCedente($cdreducont, $nrorg) {
        $cedente = $this->transmissaoFactory->getCedente($cdreducont, $nrorg);
        return $cedente;
    }
    
    public function getDadosSmtpNfse($nrorg) {
        $SMTP = $this->transmissaoFactory->getDadosSmtpNfse($nrorg);
        return $SMTP;
    }
    
    public function getXmlNotas($stringNota, $cdfilial, $nrorg) {
        $xmls = $this->transmissaoFactory->getXmlNotas($stringNota, $cdfilial, $nrorg);
        return $xmls;
    }
    
    public function getLoteNotas($rows) {
        $cdempresa = $rows[0]["CDEMPRESA"];
        $sql = "(";
        foreach($rows as $row) {
            $nrlanctonf = $row['NRLANCTONF'];
            $sql.="'$nrlanctonf',";
        }
        $sql = substr($sql,0,-1);
        $sql.=")";
        return $this->transmissaoFactory->getLoteNotas($sql, $cdempresa);
    }
    
    public function updateDataEmissao($cdfilial , $nrlanctonf, $nrorg, $dtentrsaid){
        $dtatual =  $this->formataData(DateUtil::getDataAtual(), true);
        $dtatual= DateUtil::truncateString($dtatual);
        
        $dtatual = strtotime($dtatual) > strtotime($dtentrsaid) ? $dtentrsaid : $dtatual;
        return $this->transmissaoFactory->updateDataEmissao($cdfilial , $nrlanctonf, $nrorg,$dtatual);
    }
    
     public function formataData($data, $trunc = true){
        if (!$data)
            return null;

        if (strlen($data) == 19)
            $data = DateUtil::truncateString($data);
        $data = DateUtil::getDataDeString($data);
        
        if($trunc)
            $data = DateUtil::truncateData($data);

        return $data;
    }
    
    public function updateStatus($cdfilial , $nrlanctonf, $nrorg){
        return $this->transmissaoFactory->updateStatus($cdfilial , $nrlanctonf, $nrorg);
    }
    
    public function getContareceber($cdfilial , $nrlanctonf, $nrorg){
        return $this->transmissaoFactory->getContareceber($cdfilial , $nrlanctonf, $nrorg);
    }
    
    public static function getAspUrlParameter() {
        return InstanceManager::getInstance()->getParameter('aspUrl');
    }
    
    public static function getDelphiCryptoUncripto($password, $isEncrypt = true) {
        $url = self::getAspUrlParameter() . '/comwebservice/service.asmx?wsdl';
        
        $options = ['encoding'    => 'ISO-8859-1',
                    'trace'       => true,
                    'exceptions'  => true,
                    'compression' => SOAP_COMPRESSION_ACCEPT | SOAP_COMPRESSION_GZIP];
        
        $args = [new \SoapVar($password, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema"),
                 new \SoapVar('2', XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema")];
        //2
        $cryptType = $isEncrypt ? 'GetCripto' : 'GetUnCripto';
        $params = ['rdm'    => 'Bib.rdmDBGeral',
                   'method' => $cryptType,
                   'args'   => $args];
        
        try {
            $client = new \SoapClient($url, $options);
            $result = $client->executeCOMMethod($params);
            $encryption = $result->executeCOMMethodResult();
        } catch (\SoapFault $e) {
            return 'SoapError';
        }
        return $encryption;
    }
    
}