<?php

namespace Teknisa\DF\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\Environment;
use Fiscal\Entities\Notafiscal;
use Geral\Entities\Inscrestad;
use Teknisa\DF\Factories\EmailsAdicionaisFactory;
use Teknisa\Libs\Util\NewCode;
use Teknisa\Libs\Util\Utilities;
use Teknisa\Libs\Util\DateUtil;

class EmailsAdicionaisService {
    protected $emailsAdicionaisFactory;
    protected $repositoryEmailAdicionais;
    protected $novoCodigo;
    protected $entityManager;
    protected $environment;

    public function __construct(EmailsAdicionaisFactory $emailsAdicionaisFactory, NewCode $novoCodigo, 
                                EntityManager $entityManager, Utilities $util, Environment $environment) {
        $this->emailsAdicionaisFactory = $emailsAdicionaisFactory;
        $this->novoCodigo = $novoCodigo;
        $this->entityManager = $entityManager;
        $this->util = $util;
        $this->environment = $environment;
        $this->repositoryEmailAdicionais = $this->entityManager->getRepository('\Teknisa\Model\Teknisa\Entities\\EmailAdicionais');
    }
    
    public function findEmailsAdicionais($filters, $page, $pageSize, $orderBy) {
        $conditions = $this->createQueryFiltered($filters, "E");
        
        foreach($orderBy as $idx => $order) {
            $conditions .= ' ORDER BY ' . $idx . ' ' . $order;
        }
        
        $limit  = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        
        $emails = $this->repositoryEmailAdicionais->findEmailsAdicionais($conditions, $limit, $offset);
        return $emails;
    }
    
    public function findTransportadoras($filters, $page, $pageSize, $orderBy) {
        $conditions = $this->createQueryFiltered($filters);
        
        foreach($orderBy as $idx => $order) {
            $conditions .= ' ORDER BY ' . $idx . ' ' . $order;
        }
        
        $limit  = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        $emails = $this->repositoryEmailAdicionais->findTransportadoras($conditions, $limit, $offset);
        return $emails;
    }
    
    public function saveEmailsAdicionais($emailRow){
        $connection = $this->entityManager->getConnection();
        $dateObject = DateUtil::getDataAtual();
        $date = DateUtil::truncateString($dateObject);
        $nrorg = $this->environment->getNrOrgTrab();
        $cdOperador = $this->environment->getCdOperador();
        try {
            $connection->beginTransaction();
            $email = $this->repositoryEmailAdicionais->findOneBy(array(
                    'idemail' => $emailRow['IDEMAIL']
            ));
            
            if(!$email){
                $response = $this->repositoryEmailAdicionais->adicionarEmail($emailRow, $cdOperador, $nrorg, $date);    
            } else{
                $response = $this->emailsAdicionaisFactory->editarEmail($email, $emailRow['DSEMAIL'], $cdOperador, $nrorg, $dateObject);
            }
            
            $this->entityManager->flush();
            $connection->commit();
            return $response;
        } catch(\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
    
    private function createQueryFiltered($filters, $tableAlias = null) {
        $filterSQL = '';
        foreach ($filters as $filter) {
            $name = $filter['columnName'];
            $operator = $filter['operator'];
            $values = $filter['value'];
            if (($name != '*' && !empty($values)) || $values == 0) {
                if ($operator == 'BETWEEN') {
                    if ($values[0] && $values[1]) {
                        $values = "TO_DATE( '$values[0] 00:00:00' ,'DD/MM/YYYY HH24:MI:SS')
                                AND TO_DATE( '$values[1] 23:59:59' ,'DD/MM/YYYY HH24:MI:SS')";
                    } else if ($values[0]) {
                        $operator = '>';
                        $values = " TO_DATE( '$values[0] 00:00:00' ,'DD/MM/YYYY HH24:MI:SS')";
                    } else if ($values[1]) {
                        $operator = '<';
                        $values = " TO_DATE( '$values[1] 00:00:00' ,'DD/MM/YYYY HH24:MI:SS')";
                    } else {
                        $values = "";
                    }

                    if ($values || ($values && ($values[0] || $values[1]))) {
                        if (!empty($tableAlias)) {
                            $filterSQL .= "AND $tableAlias.$name $operator $values";
                        } else {
                            $filterSQL .= "AND $name $operator $values";
                        }
                    }
                } else {

                    if (is_array($values) && $operator != 'MAPPED_LIKE_ALL') {
                        $stringValues = "('" . implode("','", $values) . "')";
                        $values = $stringValues;
                    }
                    if ($operator == 'LIKE') {
                        if (!empty($tableAlias)) {
                            $values = " UPPER('" . $values . "')";
                            $name = "UPPER($tableAlias.$name)";
                        } else {
                            $values = "UPPER('" . $values . "')";
                            $name = "UPPER($name)";
                        }
                    }
                    if ($operator == 'LIKE_ALL') {
                        if (!empty($tableAlias)) {
                            $values = "UPPER('" . $values . "')";
                        } else {
                            $values = "UPPER('" . $values . "')";
                        }

                        $columnList = explode('|', $name);
                        $likes = array();
                        foreach ($columnList as $column) {
                            $likes[] = "UPPER($column) LIKE $values";
                        }
                        $filterLikeAll = implode( ' OR ', $likes);
                        $filterSQL .= "AND ( $filterLikeAll ) ";

                    } else if ($operator == 'LIKE') {
                        $filterSQL .= "AND $name $operator $values";
                    } else if ($operator == 'IN') {
                        $filterSQL .= "AND $name $operator $values";
                    } else if ($operator == 'NOT IN') {
                        $filterSQL .= "AND $name $operator ($values)";
                    } else if ($operator == 'MAPPED_LIKE_ALL') {
                        foreach($values[0] as $index => $mappedLikeAllValues) {
                            if($index == '_ALL') {
                                $values = "UPPER('" . $mappedLikeAllValues . "')";

                                $columnList = explode('|', explode('#', $name)[0]);
                                $likes = array();
                                foreach ($columnList as $column) {
                                    if (empty($tableAlias)) {
                                        $likes[] = "UPPER($column) LIKE $values";
                                    } else {
                                        $likes[] = "UPPER($tableAlias.$column) LIKE $values";
                                    }
                                }
                                $filterLikeAll = implode(' OR ', $likes);
                                $filterSQL .= " AND ( $filterLikeAll )";
                            } else {
                                $stringValues = "('" . implode("','", $mappedLikeAllValues) . "')";
                                if (!empty($tableAlias)) {
                                    $filterSQL .= " AND $tableAlias.$index IN $stringValues ";
                                } else {
                                    $filterSQL .= " AND $index IN $stringValues ";
                                }
                            }
                        }
                    } else {
                        if (!empty($tableAlias)) {
                            $filterSQL .= "AND $tableAlias.$name $operator '$values'";
                        } else {
                            if (is_numeric($values)){
                                $filterSQL .= "AND $name $operator $values";
                            } else {
                                $filterSQL .= "AND $name $operator '$values'";
                            }
                        }
                    }
                }
            }
        }
        return $filterSQL;
    }
    
    public function removeEmailAdicional($idEmail){
        try {
            $entidade = $this->repositoryEmailAdicionais->findOneBy(array('idemail' => $idEmail));

            if($entidade){
                $this->entityManager->remove($entidade);
            }
            
            $this->entityManager->flush();
            
        } catch(\Exception $ex){
            throw $ex;
        }
    }
    
}