<?php

namespace Teknisa\DF\Services;

use Zeedhi\Framework\DTO\Request\DataSet;
use Teknisa\Libs\Util\XLS\FileLoader;
use Teknisa\Libs\Util\XLS\FileReader;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\XLS\Table;
use Teknisa\Libs\Util\XLS\Exception;
use Teknisa\DF\Services\ImportacaoRateioError;
use Geral\Entities\Filial;
use Geral\Entities\Centcust;
use Zeedhi\Framework\DataSource;
use PHPExcel_Cell;
use PHPExcel_Shared_Date;

class ImportacaoRateioService{ 
    
    const PERCENTUAL = 'P';
    const VALOR = 'V';
    
    const CDFILIAL_LENGTH = 4;
    const CCUSTO_LENGTH = 4;
    
    const IMPORT_FILE_FORMAT_ERROR = 'IMPORT_FILE_FORMAT_ERROR';
    
    protected $entityManager;
    protected $environment;
    protected $factory;
    private $titleType;
    protected $columnNames; //nome das colunas da planilha
    protected $optionalColumnNames; //nome das colunas da planilha que aceitam null
    protected $nrorg;
    
     public function readRateioTable($row, $rateioType){
        $file = $row['FILE'];
        $sheetsToLoad = array('Sheasdasdsadet1');
        $xlsFile = FileLoader::loadFromBase64($file['content'], $file['name']);

        if(!$xlsFile){
            throw new \Exception(self::IMPORT_LOAD_FILE_ERROR);
        }
        $worksheet = $xlsFile->getActiveSheet();
        $columnNames = self::getRateioColumnNames($rateioType);
        //var_dump($worksheet);
        $table = FileReader::readWorksheetTable($worksheet, $columnNames);
                
        return $table;
    }
    
    private function getRateioColumnNames($rateioType){
        $columnNames = array();
        if($rateioType == self::PERCENTUAL){
            $columnNames = array(
                'FILIAL', 'CCUSTO', 'PERCENTUAL', 'DESCRICAO'
            );
        }
        if($rateioType == self::VALOR){
            $columnNames = array(
                'FILIAL', 'CCUSTO', 'VALOR', 'DESCRICAO'
            );
        }
        return $columnNames;
    }
    
    public function formatRateio(Table $table, $rateioType){
        $columnsReference = $table->getColumnsReference();
        $tableRows = $table->getRows();
        $columnsNumber = $table->getNumberOfColumns();
        
        foreach ($tableRows as &$row) {
            for($column = 0; $column < $columnsNumber; $column++){
                $columnLetterIndex = PHPExcel_Cell::stringFromColumnIndex($column);
                $columnName = $columnsReference[$columnLetterIndex];

                $row['content'][$columnName] = self::formatValue($row['content'][$columnName], $columnName, $rateioType, $row['content']);
            
            }
        }
        return $tableRows;
    }
    
    public function validateRateioTable(Table $table, $rateioType, $dbColumnValues){
        $errors = array();
        $columnsReference = $table->getColumnsReference();
        $tableRows = $table->getRows();
        $columnsNumber = $table->getNumberOfColumns();
        foreach ($tableRows as $row) {
            for($column = 0; $column < $columnsNumber; $column++){
                $columnLetterIndex = PHPExcel_Cell::stringFromColumnIndex($column);
                $columnName = $columnsReference[$columnLetterIndex];
                $value = $row['content'][$columnName];
                $error = self::validateRateioFieldValue($value, $row['index'], $columnLetterIndex, $columnName, $rateioType, $dbColumnValues);
                if($error !== NULL){
                    array_push($errors, $error->toArray());
                }
            }
            /*$error = self::validateCentCust($row['content'], $row['index'], $dbColumnValues, $rateioType);
            if($error !== NULL) array_push($errors, $error->toArray());*/
        }
        if(count($errors) > 0){
            throw new Exception($errors, self::IMPORT_FILE_FORMAT_ERROR);
        }
    }
    
    
    private function validateRateioFieldValue($value, $row, $column, $columnName, $rateioType, $dbColumnValues){
        $error = NULL;
        
        if($value === NULL && !self::isFieldValueOptional($columnName)){
            $error = new ImportacaoTitulosError(self::IMPORT_EMPTY_CELL_VALUE_ERROR, $row, $column, $columnName);
        }
       /* else{
            if(self::isFieldToSearch($columnName, $rateioType)){
                if(!self::existsFieldOnDB($value, $dbColumnValues['CDEMPRESA'], $columnName, $rateioType)){
                    $error = new ImportacaoTitulosError(self::IMPORT_NOT_FOUND_CELL_ERROR, $row, $column, $columnName);
                }
                else{
                    if($rateioType == self::TITULOS_RECEBER && $columnName == 'COD REDUZIDO CONTA CORRENTE'
                        && $this->factory->getCodReduzidoContaCorrente($dbColumnValues['CDEMPRESA'], $value, $this->nrorg) === NULL){
                        $error = new ImportacaoTitulosError(self::IMPORT_NOT_RELATED_CELL_ERROR, $row, $column, $columnName, 'CDEMPRESA');
                    }
                }
            }
        }*/
        return $error;
    }
    
    private function validateCentCust($tableRow, $rowIndex, $dbColumnValues, $rateioType){
        $error = null;
        if($rateioType == self::TITULOS_PAGAR){
            $cdfilial = $tableRow['FILIAL DE ORIGEM'];
        }
        else{
            $cdfilial = $tableRow['FILIAL'];
        }
        $centcust = $this->factory->buscaCentCustFili($cdfilial, $dbColumnValues['CDEMPRESA'], $this->nrorg);
        if(!$centcust){
            $error = new ImportacaoTitulosError(self::IMPORT_ROW_CENTCUST_ERROR, $rowIndex);
        }
        return $error;
    }
    
    private function formatValue($value, $columnName, $rateioType, $currentRow = null){
        
        switch($columnName){
            case 'FILIAL':
                $value = self::leftZeroFill($value, self::CDFILIAL_LENGTH);
                break;
            case 'CCUSTO':
                $value = self::leftZeroFill($value, self::CDFILIAL_LENGTH);
                break;
            case 'PERCENTUAL':
            case 'VALOR':
                $value = str_replace(',', '.', $value);
                $value = floatval($value);
                break;
        }
            
        return $value;
    }
    
    private function isFieldValueOptional($columnName){
        return in_array($columnName, self::getTituloOptionalColumnNames());
    }
    
    private function leftZeroFill($string, $length){
        $string = str_pad(strval($string), $length, '0', STR_PAD_LEFT); 
        return $string;
    }
    
    private function getTituloOptionalColumnNames(){
        $optionalColumnNames = array();
        
        $optionalColumnNames = array('DESCRICAO');  
        
        return $optionalColumnNames;
    }
    
    private function isFieldToSearch($columnName, $rateioType){
        return in_array($columnName, self::getRateioColumnNames($rateioType));
    }
    
    public function buildRateioDataSet($itensRateio, $tipoRateio, $tableRows){
        $dataSet = array();
        
        
        if($tipoRateio == self::PERCENTUAL){
            foreach($itensRateio as $item){
                foreach($tableRows as $tableRow){
                    $vrcontItem = floatval($item['VRCONTITEM']);
                    $percentual = $tableRow['content']['PERCENTUAL'];
                    $vrItem =   $vrcontItem/100 * $percentual;
                    array_push($dataSet, self::buildRateioRow($item['NRSEQITPROD'], $item['CDFILIAL'], $item['NRLANCTONF'], $tipoRateio, $tableRow['content'], $vrItem));
                }
            }
        }else{
            $totalContabil = 0;
            $totalPlanilha = 0;
            foreach($itensRateio as $item){
                $totalContabil += $item['VRCONTITEM'];
            }
            foreach($itensRateio as $item){
                foreach($tableRows as $tableRow){
                    $totalRow = $tableRow['content']['VALOR'];
                    $percentRateio =  $totalRow / $totalContabil;
                    $vrItem = $item['VRCONTITEM'] * $percentRateio;
                    array_push($dataSet, self::buildRateioRow($item['NRSEQITPROD'], $item['CDFILIAL'], $item['NRLANCTONF'], $tipoRateio, $tableRow['content'], round($vrItem, 2)));
                }
            }
        }
        return $dataSet;
    }
    
    public function buildRateioRow($nrseqProd, $cdfilial, $nrlanctonf, $tipoRateio, $worksheetRow, $vrItem){
        $row = array();
        
        //Filtra no BD filial e centro de custo informados no arquivo.
        $entiFilial = Filial::findOneByCdfilial($worksheetRow['FILIAL']);
        $entiCcust = Centcust::findOneByCdcentcust($worksheetRow['CCUSTO']);
        
        //monta dataset importação de rateio
        
        $row['CDFILIAL'] = $cdfilial;
        $row['NRLANCTONF'] = $nrlanctonf;
        $row['CDFILICCUST'] = $worksheetRow['FILIAL'];
        $row['NMFILIAL'] = $entiFilial->getNmfilial();
        $row['CDCENTCUSNF'] = $worksheetRow['CCUSTO'];
        $row['NMCENTCUST'] = $entiCcust->getNmcentcust();
        $row['DSRATEIO'] = $worksheetRow['DESCRICAO'];
        $row['NRSEQITPROD'] = $nrseqProd;
        $row['VRRATENF'] = $vrItem;
        
        return $row;
    }
    
    
}